@@ -43,13 +43,24 @@ open CommonComplexOptions
 (*************  ADD/REMOVE FUNCTIONS ************)
 
 let canonize_basename name =
-  let name = String.copy name in
+  let buf = Buffer.create 100 in
   for i = 0 to String.length name - 1 do
     match name.[i] with
-    | '/' | '\\' -> name.[i] <- '_'
-    | _ -> ()
+    | '/' | '\\' -> Buffer.add_char buf '_'
+    | c -> 
+(* let remove all chars which are not in the basic ASCII set *)
+        let nc = int_of_char c in
+        try
+          Buffer.add_string buf (List.assoc nc !!filename_conversions)
+        with _ ->
+            Buffer.add_char buf (
+              match c with
+                (* for Windows *)
+                ':' | '*' | '?' | '"' | '<' | '>' | '|' | '%' -> '_'
+              | _ ->
+                  if nc > 127 || nc < 32 then '-' else c)
   done;
-  name
+  Buffer.contents buf
   
 let file_commited_name file =   
   let network = file_network file in
@@ -63,8 +74,7 @@ let file_commited_name file =
   in
   (try Unix2.safe_mkdir incoming_dir with _ -> ());
   let new_name = 
-    Filename.concat incoming_dir (canonize_basename 
-        (file_best_name file))
+    Filename.concat incoming_dir (canonize_basename  best_name)
   in
   let new_name = 
     if Sys.file_exists new_name then
@@ -91,34 +101,70 @@ name.
 let file_commit file =
   let impl = as_file_impl file in
   if impl.impl_file_state = FileDownloaded then
-    let new_name = file_commited_name file in
-    try
-      set_file_disk_name file new_name;
-      let best_name = file_best_name file in  
-      Unix32.destroy (file_fd file);
-      (* Commit the file first, and share it after... *)
-      impl.impl_file_ops.op_file_commit impl.impl_file_val new_name;
-      if not (Unix2.is_directory new_name) then 
-        ignore (CommonShared.new_shared 
-            !!incoming_directory !!incoming_directory_prio best_name new_name);
-      done_files =:= List2.removeq file !!done_files;
-      update_file_state impl FileShared;
-    with e ->
-      lprintf "Exception in file_commit: %s\n" (Printexc2.to_string e)
-      
+    let subfiles = file_files file in
+    match subfiles with
+      file :: secondary_files ->
+        let new_name = file_commited_name file in
+        (try
+            set_file_disk_name file new_name;
+            let best_name = file_best_name file in  
+            Unix32.destroy (file_fd file);
+            lprintf "destroyed\n";
+            let impl = as_file_impl file in
+            impl.impl_file_ops.op_file_commit impl.impl_file_val new_name;
+            
+            begin
+              try
+                if not (Unix2.is_directory new_name) then 
+                  ignore (CommonShared.new_shared 
+                      !!incoming_directory !!incoming_directory_prio
+                      best_name new_name);
+              with e ->
+                  lprintf "Exception %s while trying to share commited file\n"
+                    (Printexc2.to_string e);
+            end;
+            
+            update_file_state impl FileShared;
+            done_files =:= List2.removeq file !!done_files;
+            files =:= List2.removeq file !!files;
+            
+            lprintf "going to secondaries...\n";
+            List.iter (fun file ->
+(* Commit the file first, and share it after... *)
+                try
+                  let impl = as_file_impl file in
+                  impl.impl_file_ops.op_file_cancel impl.impl_file_val;
+                  update_file_state impl FileCancelled;
+                  done_files =:= List2.removeq file !!done_files;
+                  files =:= List2.removeq file !!files;
+                  
+                with e ->
+                    lprintf "Exception %s in file_commit secondaries\n" (Printexc2.to_string e);
+            ) secondary_files
+          with e ->
+              lprintf "Exception in file_commit: %s\n" (Printexc2.to_string e))
+    | _ -> assert false
+        
 let file_cancel file =
   try
     let impl = as_file_impl file in    
-    if impl.impl_file_state <> FileCancelled then begin
-        update_file_state impl FileCancelled;
-        impl.impl_file_ops.op_file_cancel impl.impl_file_val;
-        (try  Unix32.remove (file_fd file)  with e -> 
-              lprintf "Sys.remove %s exception %s\n" 
-                (file_disk_name file)
-              (Printexc2.to_string e); );
-        Unix32.destroy (file_fd file);
-        files =:= List2.removeq file !!files;
-    end
+    if impl.impl_file_state <> FileCancelled then 
+      let subfiles = file_files file in
+      if file != List.hd subfiles then
+        failwith "Cannot cancel non primary file";
+      List.iter (fun file ->
+          try
+            impl.impl_file_ops.op_file_cancel impl.impl_file_val;
+            update_file_state impl FileCancelled;
+            files =:= List2.removeq file !!files;
+          with e ->
+              lprintf "Exception %s in file_cancel\n" (Printexc2.to_string e);
+      ) subfiles;
+      (try  Unix32.remove (file_fd file)  with e -> 
+            lprintf "Sys.remove %s exception %s\n" 
+              (file_disk_name file)
+            (Printexc2.to_string e); );
+      Unix32.destroy (file_fd file);
   with e ->
       lprintf "Exception in file_cancel: %s\n" (Printexc2.to_string e)
 
@@ -128,12 +174,17 @@ let mail_for_completed_file file =
     let module M = Mailer in
     let line1 = "mldonkey has completed the download of:\r\n\r\n" in
 
-    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\nComment: %s\r\n" 
+    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\n" 
       (file_best_name file)
       (file_size file)
-      (file_comment file)
     in
     
+    let line3 = if (file_comment file) <> "" then
+	Printf.sprintf "\r\nComment: %s\r\n" (file_comment file)
+      else
+        Printf.sprintf "";
+    in
+
     let subject = if !!filename_in_subject then
         Printf.sprintf "[mldonkey] file received - %s"
         (file_best_name file)
@@ -141,7 +192,7 @@ let mail_for_completed_file file =
         Printf.sprintf "mldonkey, file received";        
     in
 
-    let line3 = if !!url_in_mail <> "" then
+    let line4 = if !!url_in_mail <> "" then
 	Printf.sprintf "\r\n%s/%s\r\n" !!url_in_mail (file_best_name file)
       else
         Printf.sprintf "";
@@ -151,7 +202,7 @@ let mail_for_completed_file file =
         M.mail_to = !!mail;
         M.mail_from = !!mail;
         M.mail_subject = subject;
-        M.mail_body = line1 ^ line2 ^ line3;
+        M.mail_body = line1 ^ line2 ^ line3 ^ line4;
       } in
     M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
 
@@ -375,8 +426,10 @@ let print_connected_servers o =
   let buf = o.conn_buf in
   networks_iter (fun r ->
       try
-       let list = network_connected_servers r in
-		if r.network_name <> "BitTorrent" then begin	
+        let list = network_connected_servers r in
+        if List.mem NetworkHasServers r.network_flags ||
+          List.mem NetworkHasSupernodes r.network_flags 
+           then begin  
        if use_html_mods o then Printf.bprintf buf "\\<div class=servers\\>";
        Printf.bprintf buf "--- Connected to %d servers on the %s network ---\n"
          (List.length list) r.network_name;
