@@ -368,6 +368,16 @@ let enable_overnet = define_option current_section
     ["enable_overnet"]
   "Set to true if you also want mldonkey to run as an overnet client (enable_donkey must be true)"
     bool_option false
+  
+let enable_kademlia = define_option current_section
+    ["enable_kademlia"]
+  "Set to true if you also want mldonkey to run as an kademlia client (enable_donkey must be true, and only experimental)"
+    bool_option false
+  
+let enable_servers = define_option current_section
+    ["enable_servers"]
+  "Set to true if you also want mldonkey to run with edonkey servers (enable_donkey must be true, and only experimental)"
+    bool_option true
 
 let enable_bittorrent = define_option current_section
     ["enable_bittorrent"]
@@ -496,7 +506,8 @@ let html_checkbox_vd_file_list = define_expert_option current_section
 let html_checkbox_search_file_list = define_expert_option current_section
     ["html_checkbox_search_file_list"] "Whether to use checkboxes in the WEB interface for search result list" bool_option false
   
-  
+let html_use_gzip = define_expert_option current_section
+    ["html_use_gzip"] "Use gzip compression on web pages" bool_option false
 
   
   
@@ -971,8 +982,20 @@ let password = define_option current_section ["password"]
     EXPERT OPTIONS
 
 *********************)
-  
-        
+
+let filename_conversions = define_expert_option current_section
+    ["filename_conversions"] 
+    "The conversions to apply on characters depending on their ASCII code"
+    (list_option (tuple2_option (int_option, string_option)))
+  [   
+    (228, "ae");
+    (246, "oe");
+    (252, "ue");
+    (223, "ss");
+    (196, "Ae");
+    (214, "Oe");
+    (220, "Ue"); 
+  ]        
 let client_timeout = define_expert_option current_section ["client_timeout"] 
   "Timeout on client connections when not queued" float_option 40.
 
