@@ -303,6 +303,7 @@ let server_print_html_header buf ext =
 		( "1", "srh", "Server number", "#" ) ; 
 		( "0", "srh", "Connect|Disconnect", "C/D" ) ; 
 		( "0", "srh", "Remove", "Rem" ) ; 
+    ( "0", "srh", "Preferred", "P" ) ; 
 		( "0", "srh", "[Hi]gh or [Lo]w ID", "ID" ) ; 
 		( "0", "srh", "Network name", "Network" ) ; 
 		( "0", "srh", "Connection status", "Status" ) ; 
@@ -326,12 +327,13 @@ let server_print s o =
     \\<td class=\\\"srb\\\" %s \\>%d\\</td\\>
     %s
     %s
+    %s
     \\<td class=\\\"sr\\\" %s\\</td\\>
     \\<td class=\\\"sr\\\"\\>%s\\</td\\>
     \\<td class=\\\"sr\\\"\\>%s\\</td\\>
     \\<td class=\\\"sr br\\\"\\>%s:%d\\</td\\>
-    \\<td class=\\\"sr ar\\\"\\>%d\\</td\\>
-    \\<td class=\\\"sr ar br\\\"\\>%d\\</td\\>
+    \\<td class=\\\"sr ar\\\"\\>%Ld\\</td\\>
+    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
     \\<td class=\\\"sr\\\"\\>%s\\</td\\>
     \\<td width=\\\"100%%\\\" class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>\n"
 	  (
@@ -364,7 +366,21 @@ let server_print s o =
         onClick=\\\"parent.fstatus.location.href='submit?q=rem+%d'\\\"\\>Rem\\</TD\\>"
       snum
       )
-      
+      (
+        if info.G.server_preferred then begin
+        Printf.sprintf
+        "\\<TD class=\\\"srb\\\" onMouseOver=\\\"mOvr(this);\\\"
+        onMouseOut=\\\"mOut(this);\\\" title=\\\"Unset preferred\\\"
+        onClick=\\\"parent.fstatus.location.href='submit?q=preferred+false+%s'\\\"\\>T\\</TD\\>"
+        (Ip.string_of_addr info.G.server_addr)
+        end else begin
+        Printf.sprintf
+        "\\<TD class=\\\"srb\\\" onMouseOver=\\\"mOvr(this);\\\"
+        onMouseOut=\\\"mOut(this);\\\" title=\\\"Set preferred\\\"
+        onClick=\\\"parent.fstatus.location.href='submit?q=preferred+true+%s'\\\"\\>F\\</TD\\>"
+        (Ip.string_of_addr info.G.server_addr)
+        end
+      )
       (if n.network_name = "Donkey" then 
         begin
             match impl.impl_server_state with
@@ -397,7 +413,7 @@ let server_print s o =
    else
 	begin
           
-        Printf.bprintf buf "[%-10s%5d] %15s:%-5d %s\n%40sUsers:%-8d Files:%-8d State:%s\n"
+        Printf.bprintf buf "[%-10s%5d] %15s:%-5d %s\n%40sUsers:%-8Ld Files:%-8Ld State:%s\n"
           (n.network_name)
           (server_num s)
           (Ip.string_of_addr info.G.server_addr) 
