@@ -308,6 +308,17 @@ module Make(M:
       let active_queue q = 
         q >= connected_sources_queue && q <= busy_sources_queue
 
+(*************************************************************************)
+(*                                                                       *)
+(*                         request_score                                 *)
+(*                                                                       *)
+(*************************************************************************)
+      
+      let request_score r = r.request_score
+      
+      let set_score_part r score =
+        r.request_score <- score
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         print_source                                  *)
@@ -412,16 +423,6 @@ module Make(M:
         Printf.bprintf buf "Next Indirect Sources: %d entries\n"
           (List.length !next_indirect_sources)
 
-(*************************************************************************)
-(*                                                                       *)
-(*                         set_score                                     *)
-(*                                                                       *)
-(*************************************************************************)
-      
-      let set_score s r score =
-        let old_score = r.request_score in
-        r.request_score <- score
-
 
 (*************************************************************************)
 (*                                                                       *)
@@ -442,7 +443,10 @@ module Make(M:
 
 (* Two things matter: the global score and the local score *)
                   if s.source_score < 3 then
-                    if r.request_score > expected_score then
+(* 2.5.25, replaced expected_score by found_score, so that sources which only
+have the file are not put in good_sources_queue, unless they have an
+  interesting chunk AND not a bad rank. *)
+                    if r.request_score > found_score then
                       if saved then
                         if 
                           r.request_time + !!min_reask_delay < last_time () 
@@ -631,7 +635,7 @@ module Make(M:
                 if connecting then begin
                     r.request_time <- last_time ();
                     if r.request_score = initial_new_source_score then
-                      set_score s r new_source_score
+                      set_score_part r new_source_score
                   end else begin
                     if r.request_time = 0 then
                       r.request_time <- last_time () - 600;
@@ -793,7 +797,8 @@ module Make(M:
 (*************************************************************************)
       
       let find_request_result s file =
-        let score =  (find_request s file).request_score in
+        let r = find_request s file in
+        let score =  r.request_score in
         if score = initial_new_source_score then File_new_source else
         if score = not_found_score then File_not_found else
         if score < not_found_score then File_possible else
@@ -813,14 +818,14 @@ module Make(M:
         if time = 0 then
           last_time () - 650
         else
-          last_time ()
+          time (* changed 2.5.24 *)
       
       let add_request s file time =
         let r =
           try
             let r = find_request s file in
             remove_from_queue s r;
-            set_score s r (if r.request_score = initial_new_source_score then
+            set_score_part r (if r.request_score = initial_new_source_score then
                 new_source_score
               else
                 r.request_score - 1);
@@ -860,7 +865,7 @@ module Make(M:
             in
             if r.request_queue < connected_sources_queue then
               remove_from_queue s r;
-            set_score s r score;
+            set_score_part r score;
             reschedule_source_for_file false s r;
         with Not_found ->
             let r = {
@@ -869,7 +874,7 @@ module Make(M:
                 request_score = possible_score;
                 request_queue = outside_queue;
               } in
-            set_score s r score;
+            set_score_part r score;
             s.source_files <- r :: s.source_files;
             reschedule_source_for_file false s r
 
@@ -920,8 +925,7 @@ we will probably query for the other file almost immediatly. *)
           ("sscore", int_to_value s.source_score ) ::
           ("addr", M.source_uid_to_value s.source_uid ) ::
           ("brand", M.source_brand_to_value s.source_brand ) ::
-          ("files", smalllist_to_value "file list" 
-              (fun s -> s)
+          ("files", smalllist_to_value (fun s -> s)
             !requests) ::
           ("age", int_to_value s.source_age ) ::
           assocs
@@ -979,7 +983,7 @@ we will probably query for the other file almost immediatly. *)
 (*************************************************************************)
       
       let add_saved_source_request s uid score time =
-        if !verbose_sources > 1 then
+        if !verbose_sources > 1 then 
           lprintf "  Request %s %d %d\n" uid score time;
         let file = 
           try
@@ -991,7 +995,7 @@ we will probably query for the other file almost immediatly. *)
               raise e 
         in
         let r = add_request s file time in
-        set_score s r score;
+        set_score_part r score;
         reschedule_source_for_file true s r;
         if !verbose_sources > 1 then
           lprintf "Put saved source %d in queue %s\n" s.source_num
@@ -1004,6 +1008,7 @@ we will probably query for the other file almost immediatly. *)
 (*************************************************************************)
       
       let value_to_source assocs =
+(*        lprintf "(1) value_to_source\n";  *)
         let get_value name conv = conv (List.assoc name assocs) in    
         
         let addr = get_value "addr" M.value_to_source_uid in
@@ -1024,6 +1029,8 @@ we will probably query for the other file almost immediatly. *)
         s.source_score <- score;
         s.source_age <- last_conn;
         s.source_brand <- brand;
+
+(*        lprintf "(2) value_to_source \n"; *)
         
         let rec iter v =
           match v with
@@ -1033,6 +1040,14 @@ we will probably query for the other file almost immediatly. *)
                   let uid = value_to_string uid in
                   let score = value_to_int score in
                   let time = value_to_int time in
+                  
+(* added in 2.5.27 to fix a bug introduced in 2.5.25 *)
+                  let score =
+                    if score land 0xffff = 0 then score asr 16 else score
+                  in
+                  
+(*                  lprintf "(3) value_to_source \n"; *)
+        
                   add_saved_source_request s uid score time
                 
                 with e -> 
@@ -1044,6 +1059,8 @@ we will probably query for the other file almost immediatly. *)
           | (StringValue _) as uid ->
               (try
                   let uid = value_to_string uid in
+(*                  lprintf "(4) value_to_source \n"; *)
+                  
                   let score = 0 in
                   let time = 0 in
                   add_saved_source_request s uid score time
@@ -1057,8 +1074,11 @@ we will probably query for the other file almost immediatly. *)
           | _ -> assert false
         
         in
+(*        lprintf "(5) value_to_source \n"; *)
         
         List.iter iter files;
+(*        lprintf "(6) value_to_source \n"; *)
+        
         raise SideEffectOption
 
 (*************************************************************************)
@@ -1203,7 +1223,7 @@ we don't care since we decided to decrease their priority ! *)
                       List.iter (fun r ->
                           if r.request_file == m then begin
                               r.request_queue <- outside_queue;
-                              set_score s r not_found_score
+                              set_score_part r not_found_score
                             end
                       ) s.source_files;
                       iter (nsources-1) q queue
@@ -1326,17 +1346,22 @@ the tail if the request could be sent. This seems thus safe. *)
         | _ -> failwith "Option should be a module"
       
       let option = define_option_class "Source"
-          (fun v -> value_to_module value_to_source v)
+          (fun v -> 
+(*            lprintf "(n) source !!\n"; *)
+            value_to_module value_to_source v)
         (fun s -> Module (source_to_value s []))
       
       let file_sources_option = ref None
       
       let attach_sources_to_file section =
+(*        lprintf "attach_sources_to_file\n"; *)
         let sources = match !file_sources_option with
             None -> 
+(*              lprintf "attaching sources this time\n"; *)
               let sources = define_option section 
                   ["sources"] ""  (listiter_option option) []
               in
+(*              lprintf "done\n"; *)
               file_sources_option := Some sources;
               sources
           | Some sources ->  sources
