@@ -710,13 +710,31 @@ let upload_credit = ref 0
 
 
 let can_write_len sock len =
-  can_write_len sock len && 
-  (let upload_rate = 
-      (if !!max_hard_upload_rate = 0 then 10000 else !!max_hard_upload_rate)
-      * 1024 in
-    not_buffer_more sock (upload_rate * (Fifo.length upload_clients)))
+  let bool1 = can_write_len sock len in
+  let upload_rate = 
+        (if !!max_hard_upload_rate = 0 then 10000 else !!max_hard_upload_rate)
+    * 1024 in
+  let bool2 = 
+    (
+(*      lprintf "upload_rate %d -> %d\n" upload_rate
+        (upload_rate * (Fifo.length upload_clients)); *)
+(* changed 2.5.24 
+Don't put in a socket more than 10 seconds of upload. 
+  *)
+    not_buffer_more sock (upload_rate * 10  (* * (Fifo.length upload_clients) *) ))
+  in
+(*  lprintf "can_write_len %b %b\n" bool1 bool2; *)
+  let b = bool1 && bool2 in
+(*  if not b then
+lprintf "bool1 %b len %d bool2 %b upload %d\n" bool1 len bool2 upload_rate;
+  *)
+  b
 
 let upload_to_one_client () =
+(*  lprintf "upload_to_one_client %d %d\n" (Fifo.length upload_clients)
+  !remaining_bandwidth;
+Fifo.iter (fun c -> lprintf "   client %d\n" (client_num c))
+upload_clients; *)
   if !remaining_bandwidth < 10000 then begin
       let c = Fifo.take upload_clients in
       client_can_upload c  !remaining_bandwidth
@@ -761,9 +779,7 @@ let next_uploads () =
       (if !!max_hard_upload_rate = 0 then 10000 * 1024
         else (maxi (!!max_hard_upload_rate - 1) 1) * 1024 );
       complete_bandwidth := !total_bandwidth;
-(*      if !verbose_upload then begin
-          lprintf "Init to %d\n" !total_bandwidth; 
-        end; *)
+(*      lprintf "Init to %d\n" !total_bandwidth;  *)
       remaining_bandwidth := 0          
     end;
   
@@ -780,16 +796,13 @@ let next_uploads () =
         lprintf "    last[%d] = %d\n" i  sent_bytes.(i)
       done; *)
       
-    end; *)
-  
+    end; *)  
   remaining_bandwidth := mini (mini (mini 
         (maxi (!remaining_bandwidth + !total_bandwidth / 10) 10000) 
       !total_bandwidth) !complete_bandwidth) 
   (!total_bandwidth - !last_sec);
   complete_bandwidth := !complete_bandwidth - !remaining_bandwidth;
-(*  if !verbose_upload then begin
-      lprintf "Remaining %d[%d]\n" !remaining_bandwidth !complete_bandwidth; 
-    end; *)
+(*  lprintf "Remaining %d[%d]\n" !remaining_bandwidth !complete_bandwidth;  *)
   sent_bytes.(!counter-1) <- !remaining_bandwidth;
   if !remaining_bandwidth > 0 then 
     next_uploads ()
