@@ -35,8 +35,6 @@ open TcpBufferedSocket
 
 exception FromGuiMessageNotImplemented
 
-let verbose_gui_decoding = ref false
-  
 (*
 gui_cut_messages is a reader for TcpBufferedSocket.t that will cut the stream
   in GUI messages, and call f on each message.
@@ -64,6 +62,11 @@ let gui_cut_messages f sock nread =
       Decoding of basic data types
 
 ****************)
+let get_int64_28 proto s pos =
+  if proto > 27 then
+    get_int64 s pos, pos+8
+  else
+    Int64.of_int (get_int s pos), pos+4
       
 let get_string s pos = 
   let len = get_int16 s pos in
@@ -258,6 +261,8 @@ let get_tag s pos =
     | 2 -> let s, pos = get_string s (pos+1) in
         String s, pos
     | 3 -> Addr (get_ip s (pos+1)), pos+5
+    | 4 -> Uint16 (get_int16 s (pos+1)), pos+3
+    | 5 -> Uint8 (get_uint8 s (pos+1)), pos+2
     | _ -> assert false
   in
   { tag_name = name; tag_value = value }, pos
@@ -493,11 +498,12 @@ let get_server proto s pos =
   let port = get_int16 s pos in
   let score = get_int s (pos+2) in
   let tags, pos = get_list get_tag s (pos+6) in
-  let nusers = get_int s pos in
-  let nfiles = get_int s (pos+4) in
-  let state, pos = get_host_state proto s (pos+8) in
+  let nusers, pos = get_int64_28 proto s pos in
+  let nfiles, pos = get_int64_28 proto s pos in
+  let state, pos = get_host_state proto s pos in
   let name, pos = get_string s pos in
   let description, pos = get_string s pos in
+  let preferred = if proto > 28 then get_bool s pos else false in
   {
     server_num = num;
     server_network = net;
@@ -512,6 +518,7 @@ let get_server proto s pos =
     server_description = description;
   server_banner = "";
     server_users = None;
+    server_preferred = preferred;
   }, pos
 
 let get_client_type s pos = 
@@ -1104,8 +1111,8 @@ let to_gui (proto : int array) opcode s =
     
     | 11 ->
         let n1 = get_int s 2 in
-        let n2 = get_int s 6 in
-        let n3 = get_int s 10 in
+        let n2, pos = get_int64_28 proto s 6 in
+        let n3, pos = get_int64_28 proto s pos in
         Server_busy (n1,n2,n3)
     
     | 12 -> 
