@@ -142,9 +142,6 @@ let number_of_sources gf =
   number_of_sources gf
     
 let number_of_active_sources gf =
-  if gf.file_all_sources > 0 then 
-    gf.file_active_sources
-  else
   let nasrcs = ref 0 in
   List.iter (fun fsrc ->
     match (client_state fsrc) with
@@ -154,9 +151,12 @@ let number_of_active_sources gf =
   !nasrcs
 
 let number_of_active_sources gf =
-  ShortLazy.compute ("number_of_active_sources", gf.file_num, 0)
-  number_of_active_sources gf
-  
+  if gf.file_all_sources > 0 then 
+    gf.file_active_sources
+  else begin
+      ShortLazy.compute ("number_of_active_sources", gf.file_num, 0)
+      number_of_active_sources gf
+    end  
   
 let net_name gf =
 	let n = network_find_by_num gf.file_network in
@@ -187,16 +187,22 @@ module Html = struct
       value onclick
   end
   
+let initialization_completed = ref false
+
 let save_config () =
   (try Unix32.flush () with e -> 
         Printf2.lprintf "Exception %s while flushing\n" (Printexc2.to_string e)
   );
-  Options.save_with_help downloads_ini;
-  CommonComplexOptions.save ();
-  networks_iter_all (fun r -> 
-      List.iter (fun opfile ->
-          Options.save_with_help opfile          
-      ) r.network_config_file);
+  if !initialization_completed then (
+    Options.save_with_help downloads_ini;
+    CommonComplexOptions.save ();
+    networks_iter_all (fun r -> 
+        List.iter (fun opfile ->
+            Options.save_with_help opfile          
+        ) r.network_config_file);
+  ) else (
+    Printf2.lprintf "Initialization not completed, bypassing state saving\n"
+  );
   ()
     
 let age_to_day date =
@@ -696,7 +702,7 @@ let html_mods_done_files buf files =
 
 \\<form name=selectForm2 action=\\\"files\\\"\\>
 \\<table cellspacing=0  cellpadding=0  width=100%%\\>\\<tr\\>
-\\<td class=downloaded width=100%%\\>Total: %d - Use 'commit' to move these completed files to the incoming directory\\</td\\>
+\\<td class=downloaded width=100%%\\>Total: %d - Use '\\<a target=\\\"fstatus\\\" href=\\\"submit?q=commit\\\"\\>commit\\</a\\>' to move these completed files to the incoming directory\\</td\\>
 \\</tr\\>\\</table\\>
 
 \\</td\\>\\</tr\\>
