@@ -25,7 +25,7 @@ open BasicSocket
   
 open CommonInteractive
 open CommonInteractive
-open CommonShared
+
 open CommonTypes
 open CommonOptions
 open CommonGlobals
@@ -279,6 +279,7 @@ let load_config () =
       CommonMessages.colour_changer ();
     end;
   networks_iter_all (fun r -> 
+(*      lprintf "(n) loading network config file\n"; *)
       List.iter (fun opfile ->
           try
             Options.load opfile
@@ -399,16 +400,19 @@ let _ =
   load_config ();
   
   add_infinite_option_timer download_sample_rate CommonFile.sample_timer;  
-  
+
+(*  lprintf "(1) CommonComplexOptions.load\n"; *)
   CommonComplexOptions.load ();
-  
+
+(*  lprintf "(2) CommonComplexOptions.load done\n"; *)
   begin
     let old_save_results = !!save_results in
     save_results =:= 0;
     CommonComplexOptions.save ();
     save_results =:= old_save_results;
   end;
-  
+
+(*  lprintf "(3) networks_iter load_complex_options\n"; *)
   networks_iter (fun r -> network_load_complex_options r);
   networks_iter_all (fun r -> 
       lprintf  (_b "Network %s %s\n") r.network_name
@@ -416,7 +420,8 @@ let _ =
           (_s "enabled") else (_s "disabled"));
       );  
   networks_iter (fun r -> 
-    network_enable r;
+(*      lprintf "(4) networks_iter enabling\n"; *)
+      network_enable r;
 (* are there drawbacks to start recover_temp unconditionally here ? *)
       if !!recover_temp_on_startup then
         network_recover_temp r;
@@ -437,20 +442,20 @@ let _ =
   add_infinite_timer 3600. hourly_timer;
   add_infinite_timer 0.1 CommonUploads.upload_download_timer;
 
-  shared_add_directory {
+  CommonShared.shared_add_directory {
     shdir_dirname = !!incoming_directory;
     shdir_priority = !!incoming_directory_prio;
     shdir_strategy = "only_directory";
     shdir_networks = [];
   };
-  List.iter shared_add_directory !!CommonComplexOptions.shared_directories;  
+  List.iter CommonShared.shared_add_directory !!CommonComplexOptions.shared_directories;  
   
   add_infinite_timer 1800. (fun timer ->
       DriverInteractive.browse_friends ());
   
   Options.prune_file downloads_ini;
 (*  Options.prune_file downloads_expert_ini; *)
-  (try CommonWeb.load_web_infos () with _ -> ());
+  add_timer 20. (fun _ -> try CommonWeb.load_web_infos () with _ -> ());
   lprintf (_b "Welcome to MLdonkey client\n"); 
   lprintf (_b "Check http://www.mldonkey.net/ for updates\n"); 
   lprintf (_b "To command: telnet %s %d\n") 
@@ -503,6 +508,7 @@ let _ =
     end;
   
   if !!create_mlsubmit then save_mlsubmit_reg ();
+  DriverInteractive.initialization_completed := true;
   DriverInteractive.save_config ();
   
   Unix32.max_cache_size := MlUnix.max_filedescs
