@@ -31,8 +31,9 @@ open Options
 module M = Gui_messages
 module C = Gui_columns
 
+let _ = Unix2.safe_mkdir CommonOptions.config_dir
 let mldonkey_gui_ini = create_options_file 
-    (Filename.concat CommonOptions.home_basedir ".mldonkey_gui.ini")
+    (Filename.concat CommonOptions.config_dir "mldonkey_gui.ini")
   
 module KeyOption = struct
     
@@ -148,31 +149,33 @@ module TabPosition = struct
   
 (** {2 Key mappings} *)
 
-let keymap_global = define_option mldonkey_gui_ini ["keymaps"; "global"]
+let mldonkey_gui_section = file_section mldonkey_gui_ini [] ""
+  
+let keymap_global = define_option mldonkey_gui_section ["keymaps"; "global"]
     "Global key bindings" 
   (list_option (tuple2_option (KeyOption.t, string_option))) []
 
-let keymap_servers = define_option mldonkey_gui_ini ["keymaps"; "servers"]
+let keymap_servers = define_option mldonkey_gui_section ["keymaps"; "servers"]
   "Key bindings for servers tab"   
   (list_option (tuple2_option (KeyOption.t, string_option))) []
 
-let keymap_downloads = define_option mldonkey_gui_ini ["keymaps"; "downloads"]
+let keymap_downloads = define_option mldonkey_gui_section ["keymaps"; "downloads"]
   "Key bindings for downloads tab" 
   (list_option (tuple2_option (KeyOption.t, string_option))) []
 
-let keymap_friends = define_option mldonkey_gui_ini ["keymaps"; "friends"]
+let keymap_friends = define_option mldonkey_gui_section ["keymaps"; "friends"]
   "Key bindings for friends tab" 
   (list_option (tuple2_option (KeyOption.t, string_option))) []
 
-let keymap_queries = define_option mldonkey_gui_ini ["keymaps"; "queries"]
+let keymap_queries = define_option mldonkey_gui_section ["keymaps"; "queries"]
   "Key bindings for queries tab" 
   (list_option (tuple2_option (KeyOption.t, string_option))) []
 
-let keymap_results = define_option mldonkey_gui_ini ["keymaps"; "results"]
+let keymap_results = define_option mldonkey_gui_section ["keymaps"; "results"]
   "Key bindings for results tab" 
   (list_option (tuple2_option (KeyOption.t, string_option))) []
 
-let keymap_console = define_option mldonkey_gui_ini ["keymaps"; "console"]
+let keymap_console = define_option mldonkey_gui_section ["keymaps"; "console"]
   "Key bindings for console tab" 
     (list_option (tuple2_option (KeyOption.t, string_option))) []
 
@@ -182,23 +185,23 @@ let add_binding map binding action =
   
 (** {2 Colors} *)
 
-let color_default = define_option mldonkey_gui_ini ["colors"; "default"]
+let color_default = define_option mldonkey_gui_section ["colors"; "default"]
     (gettext M.h_col_default) string_option "Black"
-let color_downloaded =  define_option mldonkey_gui_ini ["colors"; "downloaded"]
+let color_downloaded =  define_option mldonkey_gui_section ["colors"; "downloaded"]
     (gettext M.h_col_downloaded) string_option "Blue"
-let color_downloading =  define_option mldonkey_gui_ini ["colors"; "downloading"]
+let color_downloading =  define_option mldonkey_gui_section ["colors"; "downloading"]
     (gettext M.h_col_downloading) string_option "DarkGreen"
-let color_available =  define_option mldonkey_gui_ini ["colors"; "available"]
+let color_available =  define_option mldonkey_gui_section ["colors"; "available"]
     (gettext M.h_col_avail) string_option "Orange"
-let color_not_available =  define_option mldonkey_gui_ini ["colors"; "not_available"]
+let color_not_available =  define_option mldonkey_gui_section ["colors"; "not_available"]
     (gettext M.h_col_not_avail) string_option "Red"
-let color_connected =  define_option mldonkey_gui_ini ["colors"; "connected"]
+let color_connected =  define_option mldonkey_gui_section ["colors"; "connected"]
     (gettext M.h_col_connected) string_option "DarkGreen"
-let color_not_connected =  define_option mldonkey_gui_ini ["colors"; "not_connected"]
+let color_not_connected =  define_option mldonkey_gui_section ["colors"; "not_connected"]
     (gettext M.h_col_not_connected) string_option "Black"
-let color_connecting =  define_option mldonkey_gui_ini ["colors"; "connecting"]
+let color_connecting =  define_option mldonkey_gui_section ["colors"; "connecting"]
     M.h_col_connecting string_option "Orange"
-let color_files_listed =  define_option mldonkey_gui_ini ["colors"; "files_listed"]
+let color_files_listed =  define_option mldonkey_gui_section ["colors"; "files_listed"]
     M.h_col_files_listed string_option "Blue"
 
 (** {2 Icons} *)
@@ -206,111 +209,111 @@ let color_files_listed =  define_option mldonkey_gui_ini ["colors"; "files_liste
 let xpm_label s = ["icons" ; s]
 let xpm_mini_label s = ["mini_icons" ; s]
 
-let xpm_remove = define_option mldonkey_gui_ini (xpm_label M.o_xpm_remove)
+let xpm_remove = define_option mldonkey_gui_section (xpm_label M.o_xpm_remove)
     "" filename_option ""
-let xpm_cancel = define_option mldonkey_gui_ini (xpm_label M.o_xpm_cancel)
+let xpm_cancel = define_option mldonkey_gui_section (xpm_label M.o_xpm_cancel)
     "" filename_option ""
-let xpm_connect = define_option mldonkey_gui_ini (xpm_label M.o_xpm_connect)
+let xpm_connect = define_option mldonkey_gui_section (xpm_label M.o_xpm_connect)
     "" filename_option ""
-let xpm_disconnect = define_option mldonkey_gui_ini (xpm_label M.o_xpm_disconnect)
+let xpm_disconnect = define_option mldonkey_gui_section (xpm_label M.o_xpm_disconnect)
     "" filename_option ""
-let xpm_view_users = define_option mldonkey_gui_ini (xpm_label M.o_xpm_view_users)
+let xpm_view_users = define_option mldonkey_gui_section (xpm_label M.o_xpm_view_users)
     "" filename_option ""
-let xpm_connect_more_servers = define_option mldonkey_gui_ini (xpm_label M.o_xpm_connect_more_servers)
+let xpm_connect_more_servers = define_option mldonkey_gui_section (xpm_label M.o_xpm_connect_more_servers)
     "" filename_option ""
-let xpm_toggle_display_all_servers = define_option mldonkey_gui_ini (xpm_label M.o_xpm_toggle_display_all_servers)
+let xpm_toggle_display_all_servers = define_option mldonkey_gui_section (xpm_label M.o_xpm_toggle_display_all_servers)
     "" filename_option ""
-let xpm_remove_old_servers = define_option mldonkey_gui_ini (xpm_label M.o_xpm_remove_old_servers)
+let xpm_remove_old_servers = define_option mldonkey_gui_section (xpm_label M.o_xpm_remove_old_servers)
     "" filename_option ""
-let xpm_save = define_option mldonkey_gui_ini (xpm_label M.o_xpm_save)
+let xpm_save = define_option mldonkey_gui_section (xpm_label M.o_xpm_save)
     "" filename_option ""
-let xpm_save_all = define_option mldonkey_gui_ini (xpm_label M.o_xpm_save_all)
+let xpm_save_all = define_option mldonkey_gui_section (xpm_label M.o_xpm_save_all)
     "" filename_option ""
-let xpm_save_as = define_option mldonkey_gui_ini (xpm_label M.o_xpm_save_as)
+let xpm_save_as = define_option mldonkey_gui_section (xpm_label M.o_xpm_save_as)
     "" filename_option ""
-let xpm_edit_mp3 = define_option mldonkey_gui_ini (xpm_label M.o_xpm_edit_mp3)
+let xpm_edit_mp3 = define_option mldonkey_gui_section (xpm_label M.o_xpm_edit_mp3)
     "" filename_option ""
-let xpm_pause_resume = define_option mldonkey_gui_ini (xpm_label M.o_xpm_pause_resume)
+let xpm_pause_resume = define_option mldonkey_gui_section (xpm_label M.o_xpm_pause_resume)
     "" filename_option ""
-let xpm_get_format = define_option mldonkey_gui_ini (xpm_label M.o_xpm_get_format)
+let xpm_get_format = define_option mldonkey_gui_section (xpm_label M.o_xpm_get_format)
     "" filename_option ""
-let xpm_preview = define_option mldonkey_gui_ini (xpm_label M.o_xpm_preview)
+let xpm_preview = define_option mldonkey_gui_section (xpm_label M.o_xpm_preview)
     "" filename_option ""
-let xpm_verify_chunks = define_option mldonkey_gui_ini (xpm_label M.o_xpm_verify_chunks)
+let xpm_verify_chunks = define_option mldonkey_gui_section (xpm_label M.o_xpm_verify_chunks)
     "" filename_option ""
-let xpm_retry_connect = define_option mldonkey_gui_ini (xpm_label M.o_xpm_retry_connect)
+let xpm_retry_connect = define_option mldonkey_gui_section (xpm_label M.o_xpm_retry_connect)
     "" filename_option ""
-let xpm_add_to_friends = define_option mldonkey_gui_ini (xpm_label M.o_xpm_add_to_friends)
+let xpm_add_to_friends = define_option mldonkey_gui_section (xpm_label M.o_xpm_add_to_friends)
     "" filename_option ""
-let xpm_download = define_option mldonkey_gui_ini (xpm_label M.o_xpm_download)
+let xpm_download = define_option mldonkey_gui_section (xpm_label M.o_xpm_download)
     "" filename_option ""
-let xpm_submit_search = define_option mldonkey_gui_ini (xpm_label M.o_xpm_submit_search)
+let xpm_submit_search = define_option mldonkey_gui_section (xpm_label M.o_xpm_submit_search)
     "" filename_option ""
-let xpm_extend_search = define_option mldonkey_gui_ini (xpm_label M.o_xpm_extend_search)
+let xpm_extend_search = define_option mldonkey_gui_section (xpm_label M.o_xpm_extend_search)
     "" filename_option ""
-let xpm_local_search = define_option mldonkey_gui_ini (xpm_label M.o_xpm_local_search)
+let xpm_local_search = define_option mldonkey_gui_section (xpm_label M.o_xpm_local_search)
     "" filename_option ""
-let xpm_find_friend =  define_option mldonkey_gui_ini (xpm_label M.o_xpm_find_friend)
+let xpm_find_friend =  define_option mldonkey_gui_section (xpm_label M.o_xpm_find_friend)
     "" filename_option ""
-let xpm_remove_all_friends =  define_option mldonkey_gui_ini (xpm_label M.o_xpm_remove_all_friends)
+let xpm_remove_all_friends =  define_option mldonkey_gui_section (xpm_label M.o_xpm_remove_all_friends)
     "" filename_option ""
-let xpm_close_room =  define_option mldonkey_gui_ini (xpm_label M.o_xpm_close_room)
+let xpm_close_room =  define_option mldonkey_gui_section (xpm_label M.o_xpm_close_room)
     "" filename_option ""
-let xpm_refresh =  define_option mldonkey_gui_ini (xpm_label M.o_xpm_refresh)
+let xpm_refresh =  define_option mldonkey_gui_section (xpm_label M.o_xpm_refresh)
     "" filename_option ""
 
   
-let xpm_mini_remove = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_remove)
+let xpm_mini_remove = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_remove)
     "" filename_option ""
-let xpm_mini_cancel = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_cancel)
+let xpm_mini_cancel = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_cancel)
     "" filename_option ""
-let xpm_mini_connect = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_connect)
+let xpm_mini_connect = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_connect)
     "" filename_option ""
-let xpm_mini_disconnect = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_disconnect)
+let xpm_mini_disconnect = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_disconnect)
     "" filename_option ""
-let xpm_mini_view_users = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_view_users)
+let xpm_mini_view_users = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_view_users)
     "" filename_option ""
-let xpm_mini_connect_more_servers = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_connect_more_servers)
+let xpm_mini_connect_more_servers = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_connect_more_servers)
     "" filename_option ""
-let xpm_mini_toggle_display_all_servers = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_toggle_display_all_servers)
+let xpm_mini_toggle_display_all_servers = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_toggle_display_all_servers)
     "" filename_option ""
-let xpm_mini_remove_old_servers = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_remove_old_servers)
+let xpm_mini_remove_old_servers = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_remove_old_servers)
     "" filename_option ""
-let xpm_mini_save = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_save)
+let xpm_mini_save = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_save)
     "" filename_option ""
-let xpm_mini_save_all = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_save_all)
+let xpm_mini_save_all = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_save_all)
     "" filename_option ""
-let xpm_mini_save_as = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_save_as)
+let xpm_mini_save_as = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_save_as)
     "" filename_option ""
-let xpm_mini_edit_mp3 = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_edit_mp3)
+let xpm_mini_edit_mp3 = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_edit_mp3)
     "" filename_option ""
-let xpm_mini_pause_resume = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_pause_resume)
+let xpm_mini_pause_resume = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_pause_resume)
     "" filename_option ""
-let xpm_mini_get_format = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_get_format)
+let xpm_mini_get_format = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_get_format)
     "" filename_option ""
-let xpm_mini_preview = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_preview)
+let xpm_mini_preview = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_preview)
     "" filename_option ""
-let xpm_mini_verify_chunks = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_verify_chunks)
+let xpm_mini_verify_chunks = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_verify_chunks)
     "" filename_option ""
-let xpm_mini_retry_connect = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_retry_connect)
+let xpm_mini_retry_connect = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_retry_connect)
     "" filename_option ""
-let xpm_mini_add_to_friends = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_add_to_friends)
+let xpm_mini_add_to_friends = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_add_to_friends)
     "" filename_option ""
-let xpm_mini_download = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_download)
+let xpm_mini_download = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_download)
     "" filename_option ""
-let xpm_mini_submit_search = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_submit_search)
+let xpm_mini_submit_search = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_submit_search)
     "" filename_option ""
-let xpm_mini_extend_search = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_extend_search)
+let xpm_mini_extend_search = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_extend_search)
     "" filename_option ""
-let xpm_mini_local_search = define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_local_search)
+let xpm_mini_local_search = define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_local_search)
     "" filename_option ""
-let xpm_mini_find_friend =  define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_find_friend)
+let xpm_mini_find_friend =  define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_find_friend)
     "" filename_option ""
-let xpm_mini_remove_all_friends =  define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_remove_all_friends)
+let xpm_mini_remove_all_friends =  define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_remove_all_friends)
     "" filename_option ""
-let xpm_mini_close_room =  define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_close_room)
+let xpm_mini_close_room =  define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_close_room)
     "" filename_option ""
-let xpm_mini_refresh =  define_option mldonkey_gui_ini (xpm_mini_label M.o_xpm_mini_refresh)
+let xpm_mini_refresh =  define_option mldonkey_gui_section (xpm_mini_label M.o_xpm_mini_refresh)
     "" filename_option ""
 
 (** {2 Toolbars style} *)
@@ -340,75 +343,75 @@ let (class_tbstyle : Gtk.Tags.toolbar_style option_class) =
       define_option_class "toolbar_style" 
     value_to_tbstyle tbstyle_to_value
 
-let toolbars_style = define_option mldonkey_gui_ini
+let toolbars_style = define_option mldonkey_gui_section
     ["toolbars_style"] (gettext M.h_toolbars_style)
     class_tbstyle `BOTH
 
-let mini_toolbars = define_option mldonkey_gui_ini
+let mini_toolbars = define_option mldonkey_gui_section
     ["mini_toolbars"] "Use mini icons in toolbars"
     bool_option true
 
 (** {2 Layout} *)
 
-let servers_hpane_left = define_option mldonkey_gui_ini
+let servers_hpane_left = define_option mldonkey_gui_section
     ["layout"; "servers_hpane_left"]
     "Size in % of left part of the servers hpane"
     int_option 80
 
-let servers_vpane_up = define_option mldonkey_gui_ini
+let servers_vpane_up = define_option mldonkey_gui_section
     ["layout"; "servers_vpane_up"]
   "Size in % of upper part of the servers hpane"
     int_option 39
 
-let rooms_hpane_left = define_option mldonkey_gui_ini
+let rooms_hpane_left = define_option mldonkey_gui_section
     ["layout"; "rooms_hpane_left"]
     "Size in % of left part of the rooms hpane"
     int_option 80
 
-let downloads_hpane_left = define_option mldonkey_gui_ini
+let downloads_hpane_left = define_option mldonkey_gui_section
     ["layout"; "downloads_hpane_left"]
   "Size in % of left part of the downloads hpane"
     int_option 80
 
-let friends_hpane_left = define_option mldonkey_gui_ini
+let friends_hpane_left = define_option mldonkey_gui_section
   ["layout"; "friends_hpane_left"]
   "Size in % of left part of the friends hpane"
     int_option 30
 
-let searches_hpane_left = define_option mldonkey_gui_ini
+let searches_hpane_left = define_option mldonkey_gui_section
     ["layout"; "searches_hpane_left"]
   "Size in % of left part of the searches hpane"
     int_option 40
   
-let downloads_vpane_up = define_option mldonkey_gui_ini
+let downloads_vpane_up = define_option mldonkey_gui_section
     ["layout"; "downloads_vpane_up"]
   "Size in % of up part of the downloads vpane (downloaded/downloading)"
     int_option 50
 
-let downloads_wpane_up = define_option mldonkey_gui_ini
+let downloads_wpane_up = define_option mldonkey_gui_section
     ["layout"; "downloads_wpane_up"]
   "Size in % of up part of the downloads vpane (locations/client info)"
     int_option 75
 
-let friends_vpane_up = define_option mldonkey_gui_ini
+let friends_vpane_up = define_option mldonkey_gui_section
     ["layout"; "friends_vpane_up"]
     "Size in % of up part of the friends vpane"
     int_option 80
 
-let friends_hpane_dirs = define_option mldonkey_gui_ini
+let friends_hpane_dirs = define_option mldonkey_gui_section
     ["layout"; "friends_hpane_dirs"]
     "Size in % of the directories part of the files box"
     int_option 20
 
-let gui_width = define_option mldonkey_gui_ini
+let gui_width = define_option mldonkey_gui_section
     ["layout"; "width"]
     "Width of GUI window" int_option 600
 
-let gui_height = define_option mldonkey_gui_ini
+let gui_height = define_option mldonkey_gui_section
     ["layout"; "height"]
   "Height of GUI window" int_option 400
 
-let notebook_tab = define_option mldonkey_gui_ini
+let notebook_tab = define_option mldonkey_gui_section
     ["layout"; "tab_position"]
     "Position of tab (left, right, top, bottom)" TabPosition.t `TOP
 
@@ -419,7 +422,7 @@ let notebook_tab = define_option mldonkey_gui_ini
 (** {3 Files} *)
 
 
-let downloads_columns = define_option mldonkey_gui_ini
+let downloads_columns = define_option mldonkey_gui_section
     ["downloads_columns"] (gettext M.h_downloads_columns)
     (list_option C.File.class_column)
   [ C.Col_file_name ; C.Col_file_size ; C.Col_file_downloaded;
@@ -427,7 +430,7 @@ let downloads_columns = define_option mldonkey_gui_ini
       C.Col_file_rate ; C.Col_file_state ; 
     C.Col_file_availability ]
 
-let downloaded_columns = define_option mldonkey_gui_ini
+let downloaded_columns = define_option mldonkey_gui_section
     ["downloaded_columns"] (gettext M.h_downloaded_columns)
     (list_option C.File.class_column)
     [ C.Col_file_name ; C.Col_file_size ; 
@@ -435,13 +438,13 @@ let downloaded_columns = define_option mldonkey_gui_ini
 
 (** {3 Clients} *)
 
-let friends_columns = define_option mldonkey_gui_ini
+let friends_columns = define_option mldonkey_gui_section
     ["friends_columns"] (gettext M.h_friends_columns)
     (list_option C.Client.class_column)
     [ C.Col_client_name ; C.Col_client_kind ; 
       C.Col_client_state; ]
 
-let file_locations_columns = define_option mldonkey_gui_ini
+let file_locations_columns = define_option mldonkey_gui_section
     ["file_locations_columns"] (gettext M.h_file_locations_columns)
     (list_option C.Client.class_column)
     [ C.Col_client_name ; C.Col_client_kind ; 
@@ -449,19 +452,19 @@ let file_locations_columns = define_option mldonkey_gui_ini
 
 (** {3 Users} *)
 
-let users_columns = define_option mldonkey_gui_ini
+let users_columns = define_option mldonkey_gui_section
     ["users_columns"] "Columns of the users lists"
     (list_option C.User.class_column)
   [ C.Col_user_name ; C.Col_user_kind ;     C.Col_user_tags; ]
 
-let rooms_columns = define_option mldonkey_gui_ini
+let rooms_columns = define_option mldonkey_gui_section
     ["rooms_columns"] "Columns of the room lists"
     (list_option C.Room.class_column)
   [ C.Col_room_name ; C.Col_room_nusers ]
 
 (** {3 Servers} *)
 
-let servers_columns = define_option mldonkey_gui_ini
+let servers_columns = define_option mldonkey_gui_section
     ["server_columns"] (gettext M.h_servers_columns)
     (list_option C.Server.class_column)
     [ C.Col_server_address ; C.Col_server_state ; 
@@ -470,7 +473,7 @@ let servers_columns = define_option mldonkey_gui_ini
 
 (** {3 Results} *)
 
-let results_columns = define_option mldonkey_gui_ini
+let results_columns = define_option mldonkey_gui_section
     ["results_columns"] (gettext M.h_results_columns)
     (list_option C.Result.class_column)
     [ C.Col_result_name ; 
@@ -480,7 +483,7 @@ let results_columns = define_option mldonkey_gui_ini
 
 (** {3 Upload info} *)
 
-let shared_files_up_columns = define_option mldonkey_gui_ini
+let shared_files_up_columns = define_option mldonkey_gui_section
     ["shared_files_up_columns"] M.h_shared_files_up_columns
     (list_option C.Shared_files_up.class_column)
     [ C.Col_shared_file ; C.Col_shared_size; C.Col_shared_requests ; 
@@ -489,63 +492,63 @@ let shared_files_up_columns = define_option mldonkey_gui_ini
 
 (** {2 Others} *)
 
-let login = define_option mldonkey_gui_ini ["login"] 
+let login = define_option mldonkey_gui_section ["login"] 
     "Your login name (default is admin)" string_option "admin"
-let password = define_option mldonkey_gui_ini ["password"] 
+let password = define_option mldonkey_gui_section ["password"] 
     (gettext M.h_gui_password) string_option ""
-let port = define_option mldonkey_gui_ini ["port"] 
+let port = define_option mldonkey_gui_section ["port"] 
     (gettext M.h_gui_port) int_option 4001
-let hostname = define_option mldonkey_gui_ini ["hostname"] 
+let hostname = define_option mldonkey_gui_section ["hostname"] 
     (gettext M.h_hostname) string_option "localhost"
 
-let history = define_option mldonkey_gui_ini ["history"]
+let history = define_option mldonkey_gui_section ["history"]
     "History of connected cores"
   (list_option (tuple2_option (string_option, int_option))) []
   
 (** Profondeur max pour l'auto expand des arborescences des fichiers
    des friends. *)
-let files_auto_expand_depth = define_option mldonkey_gui_ini 
+let files_auto_expand_depth = define_option mldonkey_gui_section 
     ["files_auto_expand_depth"]
     (M.h_files_auto_expand_depth) int_option 3
 
 (** Whether we must print sizes in bytes or use G, M and k suffixes. *)
-let use_size_suffixes = define_option mldonkey_gui_ini
+let use_size_suffixes = define_option mldonkey_gui_section
     ["use_size_suffixes"]
     (M.h_use_size_suffixes) bool_option true
 
-let use_availability_height = define_option mldonkey_gui_ini ["availability_height"]
+let use_availability_height = define_option mldonkey_gui_section ["availability_height"]
   "Display the availability of each chunk as the height of the colored bar"
     bool_option true
 
-let availability_max = define_option mldonkey_gui_ini ["availability_max"]
+let availability_max = define_option mldonkey_gui_section ["availability_max"]
   "If use_availability_height is true, which availability corresponds to
   a full bar ?"
     int_option 10
   
-let use_relative_availability = define_option mldonkey_gui_ini ["relative_availability"]
+let use_relative_availability = define_option mldonkey_gui_section ["relative_availability"]
   "Calculate the % avilability ignoring already present chunks"
     bool_option true
 
-let max_client_name_len = define_option mldonkey_gui_ini
+let max_client_name_len = define_option mldonkey_gui_section
     ["max_client_name_len"] "Max length of a source name"
     int_option 18
 
-let max_result_name_len = define_option mldonkey_gui_ini
+let max_result_name_len = define_option mldonkey_gui_section
     ["max_result_name_len"] "Max length of a source name"
     int_option 70
     
-let compaction_overhead = define_option mldonkey_gui_ini 
+let compaction_overhead = define_option mldonkey_gui_section 
     ["compaction_overhead"] 
     "The percentage of free memory before a compaction is triggered"
     int_option 50
 
 (*
-let auto_resize = define_option mldonkey_gui_ini
+let auto_resize = define_option mldonkey_gui_section
     ["auto_resize"]
   "auto resize columns" bool_option true
 *)
 
-let interface_buffer = define_option mldonkey_gui_ini
+let interface_buffer = define_option mldonkey_gui_section
     ["interface_buffer"] "The size of the buffer to the core"
     int_option 10000000
 
@@ -652,18 +655,18 @@ let pixmap i = GMisc.pixmap (gdk_pix i) ()
 
 
 
-let copy_messages = define_option mldonkey_gui_ini
+let copy_messages = define_option mldonkey_gui_section
     ["copy_messages"]
   "For bundle binaries, should we directly pass structures between
     the core and the GUI (faster), or copy them (fewer bugs) " bool_option 
   true
 
-let downloads_up = define_option mldonkey_gui_ini
+let downloads_up = define_option mldonkey_gui_section
     ["downloads_up"]
   "Put the download frame up"
   bool_option true
 
-let chunk_width = define_option mldonkey_gui_ini
+let chunk_width = define_option mldonkey_gui_section
     ["chunk_width"]
     "The chunk width in pixel"
     int_option 3
