@@ -406,7 +406,7 @@ let new_server addr port=
           server_room = room_impl;
           server_name = "<unknown>";
           server_addr = addr;
-          server_nusers = 0;
+          server_nusers = Int64.zero;
           server_info = "";
           server_connection_control = new_connection_control ();
           server_sock = NoConnection;
