@@ -30,6 +30,7 @@ open CommonGlobals
 open CommonFile
 open CommonClient
 open CommonComplexOptions
+open CommonSwarming
   
 open GuiTypes
 open GuiProto
@@ -437,6 +438,7 @@ let mod_array =
     ("pille", Brand_mod_pille);
     ("morphkad", Brand_mod_morphkad);
     ("ef-mod", Brand_mod_efmod);
+      ("efmod", Brand_mod_efmod);
     ("xtreme", Brand_mod_xtreme);
     ("bionic", Brand_mod_bionic);
     ("pawcio", Brand_mod_pawcio);
@@ -477,6 +479,7 @@ let mod_array =
     ("stormit", Brand_mod_stormit);
     ("omax", Brand_mod_omax);
     ("spiders", Brand_mod_spiders);
+      ("ib\233ricaxt", Brand_mod_ibericaxt);
     ("ib\233rica", Brand_mod_iberica);
     ("stonehenge", Brand_mod_stonehenge);
     ("mison", Brand_mod_mison);
@@ -493,11 +496,38 @@ let mod_array =
     ("morphxt", Brand_mod_morphxt);
     ("ngdonkey", Brand_mod_ngdonkey);
     ("morph", Brand_mod_morph);
+      ("emule.de", Brand_mod_emulede);
+      ("aldo", Brand_mod_aldo);
     ("dm", Brand_mod_dm);
     ("lc", Brand_mod_lc);
     ("lh", Brand_mod_lh);
-    ("ice", Brand_mod_ice);
-    ("cyrex", Brand_mod_cyrex)
+      ("l!onetwork", Brand_mod_lh);
+      ("lionetwork", Brand_mod_lh);
+      ("hawkstar", Brand_mod_hawkstar);
+      ("neo mule", Brand_mod_neomule);
+      ("cyrex", Brand_mod_cyrex);
+      ("zx", Brand_mod_zx);
+      ("ackronic", Brand_mod_ackronic);
+      ("rappis", Brand_mod_rappis);
+      ("overdose", Brand_mod_overdose);
+      ("hebmule", Brand_mod_hebmule);
+      ("senfei", Brand_mod_senfei);
+      ("spoofmod", Brand_mod_spoofmod);
+      ("fusspilz", Brand_mod_fusspilz);
+      ("rocket", Brand_mod_rocket);
+      ("warezfaw", Brand_mod_warezfaw);
+      ("emusicmule", Brand_mod_emusicmule);
+      ("aideadsl", Brand_mod_aideadsl);
+      ("a i d e a d s l", Brand_mod_aideadsl);
+      ("epo", Brand_mod_epo);
+      ("kalitsch", Brand_mod_kalitsch);
+      ("raynz", Brand_mod_raynz);
+      ("serverclient", Brand_mod_serverclient);
+      ("bl4ckbird", Brand_mod_bl4ckbird);
+      ("bl4ckf0x", Brand_mod_bl4ckf0x);
+      ("candy-mule", Brand_mod_candymule);
+      ("rt", Brand_mod_rt);
+      ("ice", Brand_mod_ice)
   |]
   
 let to_lowercase s = String.lowercase s
@@ -541,7 +571,14 @@ let update_client_from_tags c tags =
       match tag.tag_name with
       | "name" -> ()
       | "version" -> ()
-      | "emule_udpports" -> ()
+      | "emule_udpports" -> 
+          for_two_int16_tag tag (fun ed2k_port kad_port ->
+(* Kademlia: we should use this client to bootstrap Kademlia *)
+              if kad_port <> 0 then
+                DonkeyProtoKademlia.Kademlia.bootstrap 
+                  c.client_ip kad_port
+          )
+
       | "emule_miscoptions1" ->
           for_int64_tag tag (fun i ->
               DonkeyProtoClient.update_emule_proto_from_miscoptions1 
@@ -600,17 +637,20 @@ let query_id ip port id =
     let module Q = DonkeyProtoUdp.QueryCallUdp in
 (*    lprintf "Ask connection from indirect client\n"; *)
     
-    let s = new_server ip port 0 in
+    let s = check_add_server ip port in
     match s.server_sock with 
       NoConnection | ConnectionWaiting _ ->
         
-        DonkeyProtoCom.udp_send (get_udp_sock ())
+(* OK, this fixes the problem with Lugdunum servers, but there should be 
+another better way, since this functionnality is still useful... 
+  
+  DonkeyProtoCom.udp_send (get_udp_sock ())
         ip (port+4)
         (DonkeyProtoUdp.QueryCallUdpReq {
             Q.ip = client_ip;
-            Q.port = !client_port;
+            Q.port = !!donkey_port;
             Q.id = id;
-          })
+          }) *) ()
     | Connection sock ->
         printf_string "[QUERY ID]";
         server_send sock (
@@ -673,11 +713,7 @@ let query_files c sock =
     end
 *)
 
-
-(* Nice to see some emule devels here... It's always possible to 
-crack a protocol, but let's try to make it as boring as possible... *)
-    
-    external hash_param : int -> int -> 'a -> int = "hash_univ_param" "noalloc"
+external hash_param : int -> int -> 'a -> int = "caml_hash_univ_param" "noalloc"
 let hash x = hash_param 10 100 x
 
 let shared_of_file file =
@@ -696,7 +732,7 @@ lprintf "       ASK VIEW FILES         \n";
           client_send c (
             let module M = DonkeyProtoClient in
             let module C = M.ViewFiles in
-            M.ViewFilesReq C.t);          
+            M.ViewFilesReq C.t);
         end
     end
 
@@ -710,6 +746,22 @@ let client_has_file c file =
         let module E = M.EmuleRequestSources in
         client_send c (M.EmuleRequestSourcesReq file.file_md4)
     end
+
+(* added in 2.5.25
+Check if the bitmap returned by a client contains a chunk that has not
+  yet been downloaded.
+  *)
+let useful_client file chunks =
+  match file.file_swarmer with
+    None -> false
+  | Some swarmer ->
+      let bitmap = Int64Swarmer.verified_bitmap swarmer in
+      let rec iter bitmap chunks i len =
+        if i = len then false else
+        if chunks.(i) && bitmap.[i] < '2' then true else
+          iter bitmap chunks (i+1) len
+      in
+      iter bitmap chunks 0 (String.length bitmap)
     
 let received_client_bitmap c md4 chunks =
   
@@ -750,14 +802,17 @@ let received_client_bitmap c md4 chunks =
 ViewFilesReq on all clients having the file to test what is
 the most widely used size for this file. Maybe create 
 different instances of the file for each proposed size ?
-
+  
 *)
       
       end else 
       chunks in
-  DonkeyOneFile.add_client_chunks c file chunks;
-  if file_state file = FileDownloading then begin
-      DonkeyOneFile.request_slot c
+  if useful_client file chunks then begin
+      DonkeySources.set_request_result c.client_source file.file_sources File_chunk;
+      DonkeyOneFile.add_client_chunks c file chunks;
+      if file_state file = FileDownloading then begin
+          DonkeyOneFile.request_slot c
+        end
     end
     
 let init_client_connection c sock =
@@ -799,6 +854,7 @@ let finish_client_handshake c sock =
   client_must_update c;
   is_banned c sock
   
+  
 let client_to_client for_files c t sock = 
   let module M = DonkeyProtoClient in
   
@@ -812,7 +868,7 @@ let client_to_client for_files c t sock =
   match t with
     M.ConnectReplyReq t ->
       printf_string "******* [CCONN OK] ********"; 
-
+      
       if DonkeySources.source_brand c.client_source then
         !activity.activity_client_overnet_successful_connections <-
           !activity.activity_client_overnet_successful_connections +1 
@@ -857,7 +913,7 @@ let client_to_client for_files c t sock =
       
       begin
         match t.CR.server_info with
-          Some (ip, port) when !!update_server_list -> safe_add_server ip port
+          Some (ip, port) -> safe_add_server ip port
         | _ -> ()
       end;
       
@@ -865,20 +921,26 @@ let client_to_client for_files c t sock =
       identify_client_brand c;
       
       init_client_connection c sock;
-
+      
       if not (register_client_hash (peer_ip sock) t.CR.md4) then
         if !!ban_identity_thieves then
           ban_client c sock "is probably using stolen client hashes";
         
-      finish_client_handshake c sock
+        finish_client_handshake c sock
   
-  | M.EmuleQueueRankingReq t 
-  | M.QueueRankReq t ->
-      c.client_rank <- t;
-      set_client_state c (Connected t);
-(* REMOVE THIS !!!
-      if t > 1000 then 
-        ban_client c sock "has an infinite queue"; *)
+  | M.EmuleQueueRankingReq rank
+  | M.QueueRankReq rank ->
+      c.client_rank <- rank;
+      set_client_state c (Connected rank);
+      if rank > !!good_client_rank then
+        List.iter (fun (file, _, _) ->
+            let s = c.client_source  in
+            let m = file.file_sources in
+            match DonkeySources.find_request_result s m with
+              File_chunk -> 
+                DonkeySources.set_request_result s m  File_found;  
+            | _ -> ()
+        ) c.client_file_queue
   
   | M.EmuleClientInfoReq t ->      
       
@@ -938,6 +1000,7 @@ let client_to_client for_files c t sock =
                   E.src_port = port;
                   E.src_server_ip = Ip.null;
                   E.src_server_port = 0;
+(* this is not very good, but what can we do ? we don't keep sources UIDs *)
                   E.src_md4 = Md4.null;
                 } :: !sources
       ) file.file_sources;
@@ -991,7 +1054,7 @@ end; *)
         | Some (file,up) -> 
             if !verbose_download then begin
                 lprintf "DonkeyClient: Clear download\n";
-	    end;
+              end;
             Int64Swarmer.clear_uploader_ranges up;
             c.client_download <- None
         | None ->
@@ -1017,8 +1080,8 @@ end; *)
                         DonkeyOneFile.add_client_chunks c file chunks) files;
 (*                DonkeyOneFile.restart_download c *)
                   with _ -> 
-                        if c.client_debug then
-                      lprintf "AvailableSlot received, but not file to download !!\n";
+                      if c.client_debug then
+                        lprintf "AvailableSlot received, but not file to download !!\n";
 (* TODO: ask for the files now *)
       end;
 (* now, we can forget we have asked for a slot *)
@@ -1084,7 +1147,7 @@ other one for unlimited sockets.  *)
         | Some (file,up) ->
             if !verbose_download then begin
                 lprintf "Slot closed during download\n";
-	    end;
+              end;
             Int64Swarmer.clear_uploader_ranges up
       end;
 (*      DonkeyOneFile.clean_current_download c; *)
@@ -1102,13 +1165,8 @@ other one for unlimited sockets.  *)
       end
   
   | M.ReleaseSlotReq _ ->
-      set_client_has_a_slot (as_client c) false;
-      client_send c (
-        let module M = DonkeyProtoClient in
-        let module Q = M.CloseSlot in
-        M.CloseSlotReq Q.t);
-      if c.client_file_queue = [] then
-        set_rtimeout sock 120.;
+      DonkeyOneFile.remove_client_slot c;
+      if c.client_file_queue = [] then set_rtimeout sock 120.;
       CommonUploads.refill_upload_slots ()
   
   | M.QueryFileReplyReq t ->
@@ -1153,13 +1211,13 @@ other one for unlimited sockets.  *)
               List.iter (fun (f,_,_) ->
                   if f == file then know_file_chunks := true
               ) c.client_file_queue;
-                
+              
               if not !know_file_chunks then
                 DonkeyProtoCom.client_send c (
                   let module M = DonkeyProtoClient in
                   M.QueryChunksReq file.file_md4);
               
-              if file.file_md4s = [||] then begin
+              if file.file_computed_md4s = [||] then begin
                   client_send c (
                     let module M = DonkeyProtoClient in
                     let module C = M.QueryChunkMd4 in
@@ -1173,7 +1231,7 @@ other one for unlimited sockets.  *)
       let module Q = M.QueryChunksReply in      
       
       received_client_bitmap c t.Q.md4 t.Q.chunks
-        
+  
   | M.QueryChunkMd4ReplyReq t ->
       begin
         let module Q = M.QueryChunkMd4Reply in
@@ -1184,12 +1242,17 @@ other one for unlimited sockets.  *)
             lprintf "MD4 FOR CHUNKS RECEIVED\n"; 
           end;
         
-        if file.file_md4s <> [||] then begin
+        if file.file_computed_md4s <> [||] then begin
             lprintf "[WARNING] Discarding Chunks Md4: already here\n";
           end else
         if file.file_nchunks = 1 then begin
             lprintf "[ERROR]: one chunk file without md4\n"; 
-            file.file_md4s <- [|file.file_md4|]
+            file.file_computed_md4s <- [|file.file_md4|];
+            match file.file_swarmer with
+              None -> ()
+            | Some swarmer ->
+                Int64Swarmer.set_verifier swarmer 
+                  (Verification [| Ed2k file.file_md4 |])
           end else
         if t.Q.chunks = [||] then begin
             lprintf "[ERROR]: empty multiple chunks message\n";
@@ -1218,7 +1281,13 @@ is checked for the file.
                 lprintf "[ERROR]: Bad list of MD4s, discarding\n"; 
               end else begin
                 file_md4s_to_register := file :: !file_md4s_to_register;
-                file.file_md4s <- md4s
+                file.file_computed_md4s <- md4s;
+                match file.file_swarmer with
+                  None -> ()
+                | Some swarmer ->
+                    Int64Swarmer.set_verifier swarmer 
+                      (Verification (Array.map (fun m -> Ed2k m) md4s))
+
               end
           
           
@@ -1256,9 +1325,9 @@ is checked for the file.
 
 (*            lprintf "Comp bloc: %d/%d\n" comp.comp_len comp.comp_total; *)
       if comp.comp_len = comp.comp_total then begin
-	if !verbose_download then begin
-          lprintf "Complete Compressed bloc received !!!!!!\n";
-	end;
+          if !verbose_download then begin
+              lprintf "Complete Compressed bloc received !!!!!!\n";
+            end;
           
           let s = String.create comp.comp_len in
           let rec iter list =
@@ -1274,9 +1343,9 @@ is checked for the file.
           assert (pos = comp.comp_len);
           if Autoconf.has_zlib then
             let s = Autoconf.zlib__uncompress_string2 s in
-	    if !verbose then begin
-            lprintf "Decompressed: %d/%d\n" (String.length s) comp.comp_len;
-	    end;
+            if !verbose then begin
+                lprintf "Decompressed: %d/%d\n" (String.length s) comp.comp_len;
+              end;
             
             DonkeyOneFile.block_received c comp.comp_md4
               comp.comp_pos s 0 (String.length s)
@@ -1314,7 +1383,7 @@ is checked for the file.
       
       DonkeyOneFile.block_received c t.Q.md4
         t.Q.start_pos t.Q.bloc_str t.Q.bloc_begin t.Q.bloc_len
-  
+
 (* Upload requests *)
   | M.ViewFilesReq t when !CommonUploads.has_upload = 0 && 
     (match !!allow_browse_share with
@@ -1348,7 +1417,7 @@ is checked for the file.
           c.client_brand <- Brand_mldonkey1;
           if Random.int 100 < 2 && !!send_warning_messages then
             client_send c (
-              M.SayReq "[AUTOMATED WARNING] Please, Update Your MLdonkey client to version 2.5-4+2");
+              M.SayReq "[AUTOMATED WARNING] Please, Update Your MLdonkey client to version 2.5-16a");
         end;
       
       begin try 	
@@ -1373,9 +1442,17 @@ is checked for the file.
             });
           DonkeySources.query_file c.client_source file.file_sources
         
-        with _ -> 
+        with Not_found -> 
             client_send c (
-              M.NoSuchFileReq md4)
+              M.NoSuchFileReq md4);
+            
+            lprintf "DUMPING File table\n";
+            Hashtbl.iter (fun md4 _ ->
+                lprintf "    %s\n" (Md4.to_string md4)
+                ) files_by_md4;
+        | e -> 
+            lprintf "Exception %s in QueryFileReq\n"
+              (Printexc.to_string e)
       end;
       
       begin
@@ -1403,14 +1480,14 @@ end else *)
             end;
           Array.iter (fun s ->
               try
-              let addr = 
+                let addr = 
                   if Ip.valid s.Q.src_ip then
                     if ip_reachable s.Q.src_ip then
                       Direct_address (s.Q.src_ip, s.Q.src_port)
                     else raise Not_found
                   else begin
 (*                      lprintf "RIA: Received indirect address\n"; *)
-                    Indirect_address (s.Q.src_server_ip, s.Q.src_server_port,
+                      Indirect_address (s.Q.src_server_ip, s.Q.src_server_port,
                         id_of_ip s.Q.src_ip)
                     end
                 in
@@ -1476,7 +1553,7 @@ end else *)
       
       let file = find_file t in
       begin
-        match file.file_md4s with
+        match file.file_computed_md4s with
           [||] -> () (* should not happen *)
         | md4s ->
             client_send c (
@@ -1496,25 +1573,31 @@ end else *)
         begin
           try
             let file = find_file t in
-            match file.file_swarmer with
-              None -> failwith "QueryChunksReq: no swarmer for file"
-            | Some swarmer ->
-                let bitmap = Int64Swarmer.verified_bitmap swarmer in
-                let chunks = 
-                  Array.init (String.length bitmap) 
-                  (fun i -> bitmap.[i] = '3')
-                in
-                client_send c (
-                  let module Q = M.QueryChunksReply in
-                  M.QueryChunksReplyReq {
-                    Q.md4 = file.file_md4;
-                    Q.chunks = chunks;
-                  });
-                DonkeySources.query_file c.client_source file.file_sources
+            let chunks =
+              match file.file_swarmer with
+                None -> [||]
+              | Some swarmer ->
+                  let bitmap = Int64Swarmer.verified_bitmap swarmer in
+                  let chunks = 
+                    Array.init (String.length bitmap) 
+                    (fun i -> bitmap.[i] = '3')
+                  in
+(* This is not very smart, as we might get banned for this request.
+TODO We should probably check if we don't know already this source...
+  *)
+                  DonkeySources.query_file c.client_source file.file_sources;
+                  chunks
+            in
+            client_send c (
+              let module Q = M.QueryChunksReply in
+              M.QueryChunksReplyReq {
+                Q.md4 = file.file_md4;
+                Q.chunks = chunks;
+              });
           with 
           | _ -> ()
         end
-        
+  
   | M.QueryBlocReq t when !CommonUploads.has_upload = 0 &&
     client_has_a_slot (as_client c) ->
       
@@ -1522,67 +1605,68 @@ end else *)
           lprintf "uploader %s(%s) ask for block\n" c.client_name
             (brand_to_string c.client_brand); 
         end;
-
+      
       let module Q = M.QueryBloc in
       let file = find_file  t.Q.md4 in
       let prio = (file_priority file) in
       let client_upload_lifetime = ref ((maxi 0 !!upload_lifetime) * 60) in
       begin
-
+        
         if !!dynamic_upload_lifetime
-           && c.client_uploaded > c.client_downloaded
-           && c.client_uploaded > Int64.mul (Int64.of_int !!dynamic_upload_threshold) zone_size
+            && c.client_uploaded > c.client_downloaded
+            && c.client_uploaded > Int64.mul (Int64.of_int !!dynamic_upload_threshold) zone_size
         then
           client_upload_lifetime :=
-              Int64.to_int 
-                (Int64.div
-                  (Int64.mul 
-                    (Int64.of_int !client_upload_lifetime)
-                    c.client_downloaded)
-                  c.client_uploaded);
+          Int64.to_int 
+            (Int64.div
+              (Int64.mul 
+                (Int64.of_int !client_upload_lifetime)
+              c.client_downloaded)
+            c.client_uploaded);
         if last_time() > c.client_connect_time + 
-          !client_upload_lifetime + 5 * prio then
+            !client_upload_lifetime + 5 * prio then
           begin
-            
+
 (* And what happens if we were downloading from this client also ? *)
             
-          disconnect_client c (Closed_for_error "Upload lifetime expired");
-          raise Not_found
-        end;
-
-      set_lifetime sock active_lifetime;
-      set_rtimeout sock !!upload_timeout;
+            disconnect_client c (Closed_for_error "Upload lifetime expired");
+            raise Not_found
+          end;
+        
+        set_lifetime sock active_lifetime;
+        set_rtimeout sock !!upload_timeout;
+        
+        let up, waiting = match c.client_upload with
+            Some ({ up_file = f } as up) when f == file ->  up, up.up_waiting
+          | Some old_up ->
+              {
+                up_file = file;
+                up_pos = Int64.zero;
+                up_end_chunk = Int64.zero;
+                up_chunks = [];
+                up_waiting = old_up.up_waiting;
+              }, old_up.up_waiting
+          | _ ->
+              {
+                up_file = file;
+                up_pos = Int64.zero;
+                up_end_chunk = Int64.zero;
+                up_chunks = [];
+                up_waiting = false;
+              }, false
+        in
+        new_chunk up t.Q.start_pos1 t.Q.end_pos1;
+        new_chunk up t.Q.start_pos2 t.Q.end_pos2;
+        new_chunk up t.Q.start_pos3 t.Q.end_pos3;
+        c.client_upload <- Some up;
+        set_client_upload (as_client c) (shared_of_file file);
+        if not waiting && !CommonUploads.has_upload = 0 then begin
+            CommonUploads.ready_for_upload (as_client c);
+            up.up_waiting <- true
+          end
+      end;
+      lprintf "QueryBloc treated\n"
       
-      let up, waiting = match c.client_upload with
-          Some ({ up_file = f } as up) when f == file ->  up, up.up_waiting
-        | Some old_up ->
-            {
-              up_file = file;
-              up_pos = Int64.zero;
-              up_end_chunk = Int64.zero;
-              up_chunks = [];
-              up_waiting = old_up.up_waiting;
-            }, old_up.up_waiting
-        | _ ->
-            {
-              up_file = file;
-              up_pos = Int64.zero;
-              up_end_chunk = Int64.zero;
-              up_chunks = [];
-              up_waiting = false;
-            }, false
-      in
-      new_chunk up t.Q.start_pos1 t.Q.end_pos1;
-      new_chunk up t.Q.start_pos2 t.Q.end_pos2;
-      new_chunk up t.Q.start_pos3 t.Q.end_pos3;
-      c.client_upload <- Some up;
-      set_client_upload (as_client c) (shared_of_file file);
-      if not waiting && !CommonUploads.has_upload = 0 then begin
-          CommonUploads.ready_for_upload (as_client c);
-          up.up_waiting <- true
-        end
-    end
-
   | M.NoSuchFileReq t ->
       begin
         try
@@ -1683,7 +1767,14 @@ let init_client sock c =
         
 let read_first_message overnet m sock =
   let module M = DonkeyProtoClient in
-  
+    
+  if !verbose_msg_clients then begin
+      lprintf "Message from incoming client";
+      CommonGlobals.print_localtime ();
+      M.print m;
+      lprintf "\n"
+    end;
+
   match m with
   
   | M.ConnectReq t ->
@@ -1803,14 +1894,12 @@ let read_first_message overnet m sock =
       
       begin
         match t.CR.server_info with
-          Some (ip, port) when !!update_server_list -> 
-            safe_add_server ip port
+          Some (ip, port) ->  safe_add_server ip port
         | None -> 
             if overnet then begin
                 lprintf "incoming Overnet client\n"; 
                 DonkeySources.set_source_brand c.client_source overnet;
               end
-        | _ -> ()
       end;
 
 (*      List.iter (fun s ->
@@ -1829,7 +1918,7 @@ let read_first_message overnet m sock =
           M.ConnectReplyReq {
             C.md4 = overnet_md4;
             C.ip = client_ip (Some sock);
-            C.port = !overnet_client_port;
+            C.port = !!overnet_port;
             C.tags = !overnet_connectreply_tags;
             C.server_info = Some (!overnet_server_ip, !overnet_server_port);
             C.left_bytes = left_bytes;
@@ -1852,7 +1941,7 @@ let read_first_message overnet m sock =
             M.ConnectReplyReq {
               C.md4 = !!client_md4;
               C.ip = client_ip (Some sock);
-              C.port = !client_port;
+              C.port = !!donkey_port;
               C.tags = !client_to_client_tags;
               C.server_info = t.CR.server_info;
               C.left_bytes = left_bytes; 
@@ -1955,7 +2044,7 @@ can be increased by AvailableSlotReq, BlocReq, QueryBlocReq
                           M.ConnectReq {
                             C.md4 = overnet_md4;
                             C.ip = client_ip None;
-                            C.port = !overnet_client_port;
+                            C.port = !!overnet_port;
                             C.tags = !overnet_connect_tags;
                             C.version = 16;
                             C.server_info = Some (!overnet_server_ip, 
@@ -1966,7 +2055,7 @@ can be increased by AvailableSlotReq, BlocReq, QueryBlocReq
                           M.ConnectReq {
                             C.md4 = !!client_md4;
                             C.ip = client_ip None;
-                            C.port = !client_port;
+                            C.port = !!donkey_port;
                             C.tags = !client_to_client_tags;
                             C.version = 16;
                             C.server_info = Some (server_ip, server_port);
@@ -1989,8 +2078,6 @@ let query_locations_reply s t =
   let module M = DonkeyProtoServer in
   let module Q = M.QueryLocationReply in
   
-  connection_ok s.server_connection_control;
-  
   try
     let file = find_file t.Q.md4 in
     let nlocs = List.length t.Q.locs in
@@ -2050,12 +2137,13 @@ let can_open_indirect_connection () =
   !DonkeySources.indirect_connections < !!max_indirect_connections
 
 let client_connection_handler overnet t event =
-  printf_string "[REMOTE CONN]";
+(*  lprintf "[REMOTE CONN]\n"; *)
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
 
       if can_open_indirect_connection () then
         begin
+(*          lprintf "+++++++++++++++++++++++++++++++++++++++++++++++\n"; *)
           (try
               let c = ref None in
               incr DonkeySources.indirect_connections;
@@ -2218,5 +2306,3 @@ a FIFO from where they are removed after 30 minutes. What about using
     	     (Printexc2.to_string e);
 	end
   )
-  
-  
\ No newline at end of file
