@@ -18,28 +18,29 @@
 *)
 
 open Int64ops
-open CommonDownloads
-
-open CommonInteractive
 open TcpBufferedSocket
 open Queues
 open Printf2
 open Md4
+open Options
+open BasicSocket
+  
+open CommonDownloads
+open CommonSwarming
+open CommonInteractive
 open CommonResult
 open CommonFile
 open CommonServer
 open CommonComplexOptions
 open CommonClient
-open Options
 open CommonTypes
-open DonkeyTypes
-open BasicSocket
 open CommonOptions
+open CommonGlobals  
+open CommonNetwork
+  
+open DonkeyTypes
 open DonkeyOptions
 open CommonOptions
-open CommonGlobals
-  
-open CommonNetwork
 
       
 (*************************************************************
@@ -136,10 +137,8 @@ let emule_info =
     E.tags = [];
   }
 
-let client_port = ref 0
 let overnet_connectreply_tags = ref ([] :  tag list)
 let overnet_connect_tags = ref ([] :  tag list)
-let overnet_client_port = ref 0
 
 let overnet_md4 = Md4.random()
 let nservers = ref 0
@@ -202,8 +201,10 @@ let interesting_clients = ref ([] : client list)
 let files_by_md4 = Hashtbl.create 127
 let find_file md4 = Hashtbl.find files_by_md4 md4
 
+(* changed 2.5.24: we store directly the size and the modification time
+in the shared_files_info *)
 let shared_files_info = (Hashtbl.create 127
-    : (string, shared_file_info) Hashtbl.t)
+    : (string * int64 * float, shared_file_info) Hashtbl.t)
 let shared_files = ref ([] : file_to_share list)
 let new_shared_files = ref [] 
   
@@ -310,7 +311,7 @@ let new_file file_state file_name md4 file_size writable =
               failwith "Zero length file ?"
         else file_size
       in
-
+      
       if file_size <> zero then
         Unix32.ftruncate64 t file_size;
       
@@ -326,7 +327,7 @@ let new_file file_state file_name md4 file_size writable =
           file_swarmer = None;
           file_nchunks = nchunks;
           file_filenames = [Filename.basename file_name, GuiTypes.noips() ];
-          file_md4s = Array.of_list md4s;
+          file_computed_md4s = Array.of_list md4s;
           file_format = FormatNotComputed 0;
           file_sources = DonkeySources.create_file_sources_manager
             (Md4.to_string md4) 
@@ -345,9 +346,14 @@ let new_file file_state file_name md4 file_size writable =
       
       file.file_sources.DonkeySources.manager_file <- (fun () -> as_file file);
       
-      (let swarmer = Int64Swarmer.create (as_file file) block_size zone_size
-        in
-        file.file_swarmer <- Some swarmer;
+      (match file_state with
+          FileShared -> ()
+        | _ ->
+            let kernel = Int64Swarmer.create_swarmer file_name file_size zone_size in
+            let swarmer = Int64Swarmer.create kernel (as_file file) block_size
+            in
+            file.file_swarmer <- Some swarmer;
+(*
         Int64Swarmer.set_writer swarmer (fun offset s pos len ->      
 (*
       lprintf "DOWNLOADED: %d/%d/%d\n" pos len (String.length s);
@@ -358,32 +364,18 @@ let new_file file_state file_name md4 file_size writable =
               Unix32.buffered_write_copy t offset s pos len
             else
               Unix32.write  t offset s pos len
-        );
-        Int64Swarmer.set_verifier swarmer (fun num begin_pos end_pos ->
-            if file.file_md4s = [||] then raise Int64Swarmer.VerifierNotReady;
-	    if !verbose then begin
-                lprintf "Md4 to compute: %d %Ld-%Ld\n" num begin_pos end_pos;
-	    end;
-            Unix32.flush_fd (file_fd file);
-            let md4 = Md4.digest_subfile (file_fd file) 
-              begin_pos (end_pos -- begin_pos) in
-            let result = md4 = file.file_md4s.(num) in
-	    if !verbose then begin
-                lprintf "Md4 computed: %s against %s = %s\n"
-                (Md4.to_string md4) 
-                (Md4.to_string file.file_md4s.(num))
-                (if result then "VERIFIED" else "CORRUPTED");
-	    end;
-            if result then begin
-                let bitmap = Int64Swarmer.verified_bitmap swarmer in
-                try ignore (String.index bitmap '3')
-                with _ -> 
+        ); *)
+            Int64Swarmer.set_verifier swarmer 
+              (if md4s = [] then VerificationNotAvailable else
+                Verification (Array.of_list (List.map (fun md4 -> Ed2k md4) md4s))
+            );
+            Int64Swarmer.set_verified swarmer (fun nblocks num ->
+                if nblocks = 1 then begin
                     new_shared_files := file :: !new_shared_files;
-                    file_must_update file;
-              end;
-            result)
-        );
-        
+                    file_must_update file
+                  end)
+      );
+      
       update_best_name file;
       file_add file_impl file_state;
       Heap.set_tag file tag_file;
@@ -422,10 +414,14 @@ let is_black_address ip port =
           true))
   
 let new_server ip port score = 
-  let key = (ip, port) in
+  let key = (ip) in
   try
-    Hashtbl.find servers_by_key key
-  with _ ->
+    let found = Hashtbl.find servers_by_key key in
+(* Is updating port to the most recent value the correct thing to do ?
+   PlasmaHH says they're legitimate servers switching ports :( *)
+    found.server_port <- port;
+    found
+  with Not_found ->
       let rec s = { 
         server_server = server_impl;
         server_next_udp = last_time ();
@@ -439,15 +435,15 @@ let new_server ip port score =
         server_connection_control = new_connection_control ();
         server_score = score;
         server_tags = [];
-        server_nfiles = 0;
-        server_nusers = 0;
+        server_nfiles = Int64.zero;
+        server_nusers = Int64.zero;
           server_max_users = 0;
         server_name = "";
         server_description = "";
         server_banner = "";
         server_users = [];
         server_master = false;
-        server_mldonkey = false;
+        server_preferred = false;
         server_last_message = 0;
         server_queries_credit = 0;
         server_waiting_queries = [];
@@ -469,11 +465,11 @@ let new_server ip port score =
       s      
 
 let find_server ip port =
-  let key = (ip, port) in
+  let key = (ip) in
   Hashtbl.find servers_by_key key  
 
 let remove_server ip port =
-  let key = (ip, port) in
+  let key = (ip) in
   let s = Hashtbl.find servers_by_key key in
   try
     Hashtbl.remove servers_by_key key;
@@ -496,7 +492,7 @@ let dummy_client =
 (*      client_sock = NoConnection; *)
       client_ip = Ip.null;
       client_md4 = Md4.null;
-      client_last_filereqs = 0;
+(*      client_last_filereqs = 0; *)
 (*      client_chunks = [||]; *)
       client_download = None;
 (*      client_block = None; *)
@@ -555,7 +551,7 @@ let create_client key =
 (*      client_sock = NoConnection; *)
       client_ip = Ip.null;
       client_md4 = Md4.null;
-      client_last_filereqs = 0;
+(*      client_last_filereqs = 0; *)
 (*      client_chunks = [||]; *)
       client_download = None;
 (*      client_block = None; *)
@@ -607,14 +603,6 @@ let new_client key =
       create_client key
 
 let create_client = ()
-      
-      (*
-let new_client_with_num key num =
-  try
-    { dummy_client with client_kind = key }  
-  with _ ->
-      create_client key
-        *)
 
 let find_client_by_key key = 
   H.find clients_by_kind { dummy_client with client_kind = key }
@@ -630,23 +618,8 @@ let friend_add c =
       
 let set_client_name c name md4 =
   if name <> c.client_name || c.client_md4 <> md4 then begin
-(*      hashtbl_remove clients_by_name c.client_name c;   *)
-
       c.client_name <- name;
       c.client_md4 <- md4;
-      
-      (*
-      if not (Hashtbl.mem clients_by_name c.client_name) then
-        Hashtbl.add clients_by_name c.client_name c;
-*)
-
-      (*
-      try      
-        let kind = Indirect_location (name, md4) in
-        let cc = H.find clients_by_kind { dummy_client with client_kind = kind } in
-        if cc != c && client_type cc land client_friend_tag <> 0 then
-          friend_add c
-      with _ -> () *)
     end
     
 exception ClientFound of client
@@ -782,7 +755,6 @@ let left_bytes = "MLDK"
 let overnet_server_ip = ref Ip.null
 let overnet_server_port = ref 0
 
-        
 let brand_to_string b =
   match b with
     Brand_unknown -> "unknown"
@@ -874,6 +846,31 @@ let brand_mod_to_string b =
   | Brand_mod_morphxt -> "MorphXT"
   | Brand_mod_ngdonkey -> "ngdonkey"
   | Brand_mod_cyrex -> "Cyrex"
+  | Brand_mod_hawkstar -> "Hawkstar"
+  | Brand_mod_neomule -> "Neo Mule"
+  | Brand_mod_aldo -> "aldo"
+  | Brand_mod_emulede -> "emule.de"
+  | Brand_mod_zx -> "zx"
+  | Brand_mod_ibericaxt -> "ib\233ricaxt"
+  | Brand_mod_candymule -> "candy-mule"
+  | Brand_mod_ackronic -> "ackronic"
+  | Brand_mod_rappis -> "rappis"
+  | Brand_mod_overdose -> "overdose"
+  | Brand_mod_hebmule -> "hebmule"
+  | Brand_mod_senfei -> "senfei"
+  | Brand_mod_spoofmod -> "spoofmod"
+  | Brand_mod_fusspilz -> "fusspilz"
+  | Brand_mod_rocket -> "rocket"
+  | Brand_mod_warezfaw -> "warezfaw"
+  | Brand_mod_emusicmule -> "emusicmule"
+  | Brand_mod_aideadsl -> "aideadsl"
+  | Brand_mod_epo -> "epo"
+  | Brand_mod_kalitsch -> "kalitsch"
+  | Brand_mod_raynz -> "raynz"
+  | Brand_mod_serverclient -> "serverclient"
+  | Brand_mod_bl4ckbird -> "bl4ckbird"
+  | Brand_mod_bl4ckf0x -> "bl4ckf0x"
+  | Brand_mod_rt -> "rt"
     
 
 let check_result r tags =
@@ -885,6 +882,7 @@ let check_result r tags =
                 (match tag.tag_value with
                   String s -> s
                 | Uint64 i | Fint64 i -> Int64.to_string i
+                | Uint16 n | Uint8 n -> string_of_int n
                 | Addr _ -> "addr");
               lprintf "\n";
           ) tags;
