@@ -61,7 +61,8 @@ let result_name r =
 
       
 let reconnect_all file =
-  DonkeyOvernet.recover_file file;
+  DonkeyProtoOvernet.Overnet.recover_file file;
+  DonkeyProtoKademlia.Kademlia.recover_file file;
   
 (* This is expensive, no ? *)
 (*  DonkeySources.reschedule_sources file; *)
@@ -76,7 +77,9 @@ let forget_search s =
   if !xs_last_search = s.search_num then begin
       xs_last_search := (-1);
       xs_servers_list := [];
-    end
+    end;
+  DonkeyProtoKademlia.Kademlia.forget_search s;
+  DonkeyProtoOvernet.Overnet.forget_search s
   
   
 let load_server_met filename =
@@ -161,7 +164,8 @@ let really_query_download filenames size md4 location old_file absents =
     (fun name -> name , { ips=[]; nips=0}) filenames);
   update_best_name file;
 
-  DonkeyOvernet.recover_file file;
+  DonkeyProtoOvernet.Overnet.recover_file file;
+  DonkeyProtoKademlia.Kademlia.recover_file file;
   
   current_files := file :: !current_files;
 (*  !file_change_hook file; *)
@@ -314,7 +318,7 @@ let import_config dirname =
       | { tag_name = "name"; tag_value = String s } ->
           login =:=  s
       | { tag_name = "port"; tag_value = Uint64 v } ->
-          port =:=  Int64.to_int v
+          donkey_port =:=  Int64.to_int v
       | _ -> ()
   ) ct;
 
@@ -406,7 +410,7 @@ let recover_md4s md4 =
   match file.file_swarmer with
     None -> ()
   | Some swarmer ->
-      Int64Swarmer.verify_all_blocks swarmer false
+      Int64Swarmer.verify_all_chunks swarmer false
   
   (*
   if file.file_chunks <> [||] then
@@ -435,7 +439,7 @@ let parse_donkey_url url =
   | "ed2k://" :: "server" :: ip :: port :: _
   | "server" :: ip :: port :: _ ->
       let ip = Ip.of_string ip in
-      let s = add_server ip (int_of_string port) in
+      let s = check_add_server ip (int_of_string port) in
       server_connect (as_server s.server_server);
       true 
   | "ed2k://" :: "friend" :: ip :: port :: _
@@ -465,7 +469,7 @@ let commands = [
         let ip = Ip.from_name ip in
         let port = int_of_string port in
         
-        let s = add_server ip port in
+        let s = force_add_server ip port in
         Printf.bprintf buf "New server %s:%d\n" 
           (Ip.to_string ip) port;
         ""
@@ -596,10 +600,6 @@ let commands = [
                         let old_downloaded = Int64Swarmer.downloaded swarmer in
                         Int64Swarmer.set_present swarmer segments;
                         let new_downloaded = Int64Swarmer.downloaded swarmer in
-                        if new_downloaded > old_downloaded then
-                          add_file_downloaded file.file_file
-                            (new_downloaded -- old_downloaded);
-                        
                         Printf.bprintf buf "Recovered %Ld bytes for %s\n"
                           (new_downloaded -- old_downloaded) 
                         (file_best_name file)
@@ -609,6 +609,18 @@ let commands = [
         ) args;
         ""
     ) , "<f1> < f2> ... :\t\t\ttry to recover these files at byte level";
+
+    "preferred", Arg_two (fun arg1 arg2 o ->
+        let preferred = bool_of_string arg1 in
+        let ip = Ip.of_string arg2 in
+        Hashtbl.iter (fun ip_s s ->
+            if ip_s = ip then begin
+              s.server_preferred <- preferred;
+              server_must_update s;
+            end
+        ) servers_by_key;
+        "ok"
+    ), "<true/false> <ip> : set the server with this IP has preferred";
     
     "bs", Arg_multiple (fun args o ->
         List.iter (fun arg ->
@@ -619,7 +631,7 @@ let commands = [
     ), "<ip1> <ip2> ... :\t\t\tadd these IPs to the servers black list";
     
     "port", Arg_one (fun arg o ->
-        port =:= int_of_string arg;
+        donkey_port =:= int_of_string arg;
         "new port will change at next restart"),
     "<port> :\t\t\t\tchange connection port";
     
@@ -785,7 +797,7 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+renameTextOut+'\\\"';
     ), ":\t\t\t\tsend the list of connected servers to the redirector";
   
   ]
-  
+          
 let _ =
   register_commands commands;
   file_ops.op_file_resume <- (fun file ->
@@ -797,13 +809,49 @@ DonkeySources.recompute_ready_sources ()        *)
       ()
   );
   file_ops.op_file_pause <- (fun file -> ()  );
+  file_ops.op_file_print_sources_html_header <- (fun file buf info ->
+      
+      html_mods_table_header buf "sourcesTable" "sources al" ([ 
+          ( "1", "srh ac", "Client number (click to add as friend)", "Num" ) ; 
+          ( "0", "srh", "[A] = Active download from client", "A" ) ; 
+          ( "0", "srh", "Client state", "CS" ) ; 
+          ( "0", "srh", "Client name", "Name" ) ; 
+          ( "0", "srh", "Client brand", "CB" ) ; 
+        ] @
+          (if !!emule_mods_count then [( "0", "srh", "eMule MOD", "EM" )] else [])
+        @ [
+          ( "0", "srh", "Overnet [T]rue, [F]alse", "O" ) ; 
+          ( "0", "srh", "Connection [I]ndirect, [D]irect", "C" ) ; 
+          ( "0", "srh br", "IP address", "IP address" ) ; 
+          ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ; 
+          ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ; 
+          ( "1", "srh ar", "Your queue rank on this client", "Rnk" ) ; 
+          ( "1", "srh ar br", "Source score", "Scr" ) ; 
+          ( "1", "srh ar", "Last ok (minutes)", "LO" ) ; 
+          ( "1", "srh ar", "Last try (minutes)", "LT" ) ; 
+          ( "1", "srh ar br", "Next try (minutes)", "NT" ) ; 
+          ( "0", "srh", "Has a slot [T]rue, [F]alse", "H" ) ; 
+          ( "0", "srh br", "Banned [T]rue, [F]alse", "B" ) ; 
+          ( "1", "srh ar", "Requests sent", "RS" ) ; 
+          ( "1", "srh ar", "Requests received", "RR" ) ; 
+          ( "1", "srh ar br", "Connected time (minutes)", "CT" ) ; 
+          ( "0", "srh br", "Client MD4", "MD4" ) ; 
+          ( "0", "srh", "Chunks (absent|partial|present|verified)", 
+            (colored_chunks 
+                (Array.init (String.length info.G.file_chunks)
+                (fun i -> ((int_of_char info.G.file_chunks.[i])-48)))) ) ; 
+          ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
+                (let fc = ref 0 in (String.iter (fun s -> if s = '2' then incr fc) 
+                  info.G.file_chunks );!fc ))) ])
+  );
+  
   file_ops.op_file_commit <- (fun file new_name ->
       
 (*      DonkeyStats.save_download_history file; *)
       
       if not (List.mem file.file_md4 !!old_files) then
         old_files =:= file.file_md4 :: !!old_files;
-      lprintf "REMEMBER SHARE FILE INFO %s\n" new_name; 
+      DonkeyShare.remember_shared_info file new_name;
       
 (**************************************************************
 
@@ -815,7 +863,7 @@ file.---------> to be done urgently
 
 ***************************************************************)
       
-      DonkeyShare.remember_shared_info file new_name
+      unshare_file file;
   );
   network.op_network_connected <- (fun _ ->
     !nservers > 0  
@@ -908,6 +956,7 @@ let _ =
           P.server_description = s.server_description;
           P.server_banner = s.server_banner;
           P.server_users = None;
+          P.server_preferred = s.server_preferred;
         }
       else raise Not_found
   )
@@ -925,6 +974,7 @@ let _ =
         P.user_server = u.user_server.server_server.impl_server_num;
       }
   )
+
 let string_of_client_addr c =
   try match c.client_source.DonkeySources.source_sock with
       Connection sock -> (Ip.to_string (peer_ip sock)) 
@@ -955,8 +1005,7 @@ let _ =
         P.client_upload = 
         (match c.client_upload with
             Some cu -> Some (file_best_name cu.up_file)
-          | None -> None);
-        
+          | None -> None);        
       }
   );
   client_ops.op_client_debug <- (fun c debug ->
@@ -964,8 +1013,8 @@ let _ =
 
 let _ =
   network.op_network_connect_servers <- (fun _ ->
-      force_check_server_connections true
-  )
+      force_check_server_connections true  )
+
 let disconnect_server s r =
   match s.server_sock with
     NoConnection -> ()
@@ -974,7 +1023,7 @@ let disconnect_server s r =
       s.server_sock <- NoConnection
   | Connection sock ->
       TcpBufferedSocket.shutdown sock r
-      
+
 let _ =
   server_ops.op_server_remove <- (fun s ->
       DonkeyComplexOptions.remove_server s.server_ip s.server_port
@@ -1047,13 +1096,20 @@ let _ =
       if !!keep_cancelled_in_old_files &&
         not (List.mem file.file_md4 !!old_files) then
         old_files =:= file.file_md4 :: !!old_files;
+      DonkeyProtoOvernet.Overnet.cancel_recover_file file;
+      DonkeyProtoKademlia.Kademlia.cancel_recover_file file;
   );
   file_ops.op_file_comment <- (fun file ->
       Printf.sprintf "ed2k://|file|%s|%Ld|%s|" 
         (file_best_name file)
       (file_size file)
       (Md4.to_string file.file_md4)
-  )
+  );
+  file_ops.op_file_files <- (fun file impl -> 
+      match file.file_swarmer with
+        None -> [CommonFile.as_file impl]
+      | Some swarmer ->
+          Int64Swarmer.subfiles swarmer)
   
 let _ =
   network.op_network_extend_search <- (fun s e ->
@@ -1213,7 +1269,7 @@ lprint_newline ();
             if qfiles <> [] then begin
                 try
                   let _, qchunks,_ = List.find (fun (qfile, _,_) ->
-                        qfile = (as_file_impl file).impl_file_val) qfiles in
+                        qfile == (as_file_impl file).impl_file_val) qfiles in
                   let tc = ref 0 in
                   Printf.bprintf buf "%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%d\\</td\\>" 
                     (CommonFile.colored_chunks (Array.init (Array.length qchunks) 
@@ -1241,7 +1297,7 @@ lprint_newline ();
           | Some _ ->  ( 
                 let qfiles = c.client_file_queue in
                 let (qfile, qchunks,_) =  List.hd qfiles in
-                if (qfile = (as_file_impl file).impl_file_val) then begin
+                if (qfile == (as_file_impl file).impl_file_val) then begin
 	          Printf.bprintf buf "[Donkey%6d] Name  : %-27s IP   : %-20s"
 	            (client_num c)
 	            (shorten c.client_name 20)
@@ -1274,7 +1330,7 @@ lprint_newline ();
           | Some _ ->  ( 
                 let qfiles = c.client_file_queue in
                 let (qfile, qchunks,_) =  List.hd qfiles in
-                if (qfile = (as_file_impl file).impl_file_val) then begin
+                if (qfile == (as_file_impl file).impl_file_val) then begin
                     let i = (client_info (as_client c)) in
                     
                     Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\" onMouseOut=\\\"mOut(this);\\\" 
@@ -1318,17 +1374,15 @@ let _ =
       let s = u.user_server in
       DonkeyUdp.add_user_friend s u
   )
-
   
 let _ =
   shared_ops.op_shared_unshare <- (fun file ->
-      match file.file_shared with
-        None -> ()
-      | Some s -> 
-          file.file_shared <- None;
-          decr nshared_files;
-          (try Unix32.close  (file_fd file) with _ -> ());
-          try Hashtbl.remove files_by_md4 file.file_md4 with _ -> ()
+      lprintf "************ UNSHARE FILE ************\n";
+      unshare_file file;
+      
+(* Should we or not ??? *)
+      if file_state file = FileShared then
+        try Hashtbl.remove files_by_md4 file.file_md4 with _ -> ();
   );
   shared_ops.op_shared_info <- (fun file ->
    let module T = GuiTypes in
@@ -1366,5 +1420,6 @@ let _ =
   
   
   network.op_network_add_server <- (fun ip port ->
-      as_server (new_server (Ip.ip_of_addr ip) port 0).server_server
+      let s = DonkeyComplexOptions.force_add_server (Ip.ip_of_addr ip) port in
+      as_server s.server_server
   )
