@@ -272,9 +272,34 @@ type brand_mod =
 | Brand_mod_blackmule
 | Brand_mod_morphxt
 | Brand_mod_ngdonkey
+| Brand_mod_hawkstar
+| Brand_mod_neomule
 | Brand_mod_cyrex
+| Brand_mod_aldo
+| Brand_mod_emulede
+| Brand_mod_zx
+| Brand_mod_ibericaxt
+| Brand_mod_candymule
+| Brand_mod_ackronic
+| Brand_mod_rappis
+| Brand_mod_overdose
+| Brand_mod_hebmule
+| Brand_mod_senfei
+| Brand_mod_spoofmod
+| Brand_mod_fusspilz
+| Brand_mod_rocket
+| Brand_mod_warezfaw
+| Brand_mod_emusicmule
+| Brand_mod_aideadsl
+| Brand_mod_epo
+| Brand_mod_kalitsch
+| Brand_mod_raynz
+| Brand_mod_serverclient
+| Brand_mod_bl4ckbird
+| Brand_mod_bl4ckf0x
+| Brand_mod_rt
 
-let brand_mod_count = 73
+let brand_mod_count = 98
 
 type source_uid = 
   Direct_address of Ip.t * int
@@ -288,7 +313,7 @@ module DonkeySources = CommonSources.Make(struct
       
       open Options
 
-      type source_brand = bool (* overnet or not *)
+      type source_brand = bool
       let value_to_source_brand = value_to_bool
       let source_brand_to_value = bool_to_value
       let dummy_source_brand = false
@@ -343,8 +368,8 @@ type server = (*[]*){
     mutable server_connection_control : connection_control;
     mutable server_score : int;
     mutable server_tags : CommonTypes.tag list;
-    mutable server_nusers : int;
-    mutable server_nfiles : int;
+    mutable server_nusers : int64;
+    mutable server_nfiles : int64;
     mutable server_max_users : int;
     mutable server_name : string;
     mutable server_description : string;
@@ -352,7 +377,7 @@ type server = (*[]*){
     mutable server_users: user list;
     mutable server_next_udp : int;
     mutable server_master : bool;
-    mutable server_mldonkey : bool;
+    mutable server_preferred : bool;
     mutable server_last_message : int; (* used only by mldonkey server *)
 
     mutable server_id_requests : file option Fifo.t;
@@ -469,7 +494,7 @@ and client = {
     mutable client_checked : bool;
     mutable client_connected : bool;
 (* statistics *)
-    mutable client_last_filereqs : int;    
+(*    mutable client_last_filereqs : int;     *)
     mutable client_downloaded : Int64.t;
     mutable client_uploaded : Int64.t;
     mutable client_brand : brand;
@@ -580,7 +605,7 @@ and file = {
 (*    mutable file_absent_chunks : (int64 * int64) list; *)
     mutable file_filenames : (string * GuiTypes.ips_list) list;
 (*    mutable file_nsources : int; *)
-    mutable file_md4s : Md4.t array;
+    mutable file_computed_md4s : Md4.t array; 
     mutable file_format : format;
 (*    mutable file_available_chunks : int array; *)
     mutable file_shared : file CommonShared.shared_impl option;
