@@ -17,10 +17,15 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open Autoconf
 open Int64ops
 open Xml_types
 open Printf2
 open Md4
+open Options
+open BasicSocket
+  
+open CommonDownloads
 open CommonSearch
 open CommonGlobals
 open CommonUser
@@ -32,14 +37,12 @@ open CommonTypes
 open CommonComplexOptions
 open CommonFile
 open CommonInteractive
-open Options
+open CommonHosts
+  
 open GnutellaTypes
 open GnutellaOptions
 open GnutellaGlobals
 open GnutellaComplexOptions
-open BasicSocket
-open CommonHosts
-open Autoconf
 open GnutellaProtocol
 
 (* Don't share files greater than 10 MB on Gnutella and limit to 200 files. 
@@ -201,12 +204,26 @@ let _ =
           as_client c
       ) file.file_clients
   );
+  file_ops.op_file_files <- (fun file impl -> 
+      match file.file_swarmer with
+        None -> [CommonFile.as_file impl]
+      | Some swarmer ->
+          Int64Swarmer.subfiles swarmer)
+;
   file_ops.op_file_active_sources <- file_ops.op_file_all_sources;
   file_ops.op_file_recover <- (fun file ->
       GnutellaServers.recover_file file;
       List.iter (fun c ->
           GnutellaServers.get_file_from_source c file
       ) file.file_clients
+  );
+  file_ops.op_file_print_sources_html_header <- (fun file buf info ->      
+      html_mods_table_header buf "sourcesTable" "sources al" [ 
+        ( "1", "srh br ac", "Client number", "Num" ) ; 
+        ( "0", "srh br", "Client Name", "Name" ) ; 
+        ( "0", "srh", "IP address", "IP address" ) ; 
+        ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ; 
+        ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ; ]      
   )
 
   
@@ -265,6 +282,7 @@ let _ =
           P.server_description = "";
           P.server_users = None;
           P.server_banner = "";
+          P.server_preferred = false;
           } else
         raise Not_found
   );
