@@ -739,13 +739,14 @@ static int ml_gethostbyname(char *hostname)
   return 1;
 }
 
-extern value alloc_inet_addr(uint32 a);
+
+extern value alloc_inet_addr(struct in_addr * a);
 
 static value alloc_one_addr(char volatile *a)
 {
   struct in_addr addr;
   memmove (&addr, (char*)a, entry_h_length);
-  return alloc_inet_addr(addr.s_addr);
+  return alloc_inet_addr(&addr);
 }
 
 static void store_in_job(value job_v)
@@ -771,7 +772,7 @@ static void store_in_job(value job_v)
   End_roots();
 }
 
-#if !defined(HAVE_LIBPTHREAD) || !(HAS_GETHOSTBYNAME_R || GETHOSTBYNAME_IS_REENTRANT)
+#if !defined(HAVE_PTHREAD) || !(HAS_GETHOSTBYNAME_R || GETHOSTBYNAME_IS_REENTRANT)
 
 value ml_ip_job_start(value job_v)
 {
